# -*- coding: utf-8 -*-
"""
tag event
"""
from flybirds.core.global_context import GlobalContext
from flybirds.utils import launch_helper
from flybirds.utils import flybirds_log as log


class OnBefore:  # pylint: disable=too-few-public-methods
    """
    before tag
    """

    name = "OnBefore"
    order = 5

    @staticmethod
    def can(context, tag):
        return True

    @staticmethod
    def run(context, tag):
        """
        exe before
        """
        log.info(tag)
        # hook extend by tester
        before_tag_extend = launch_helper.get_hook_file("before_tag_extend")
        if before_tag_extend is not None:
            before_tag_extend(context, tag)


class OnAfter:  # pylint: disable=too-few-public-methods
    """
    after tag
    """

    name = "OnAfter"
    order = 100

    @staticmethod
    def can(context, tag):
        return True

    @staticmethod
    def run(context, tag):
        """
        exe after tag
        """
        # hook extend by tester
        after_tag_extend = launch_helper.get_hook_file("after_tag_extend")
        if after_tag_extend is not None:
            after_tag_extend(context, tag)


# add tag event into global processor
var = GlobalContext.join("before_tag_processor", OnBefore, 1)
var1 = GlobalContext.join("after_tag_processor", OnAfter, 1)



# -*- coding: utf-8 -*-
"""
Step implement of page adjust.
"""
import re

import flybirds.core.global_resource as gr
import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_swipe as ps
import flybirds.utils.dsl_helper as dsl_helper


def swipe_to_ele(context, param1, param2, param3):
    poco_instance = gr.get_value("pocoInstance")

    param1_dict = dsl_helper.params_to_dic(param1)
    container_dsl_str = param1_dict["selector"]
    container_optional = {}
    if "path" in param1_dict.keys():
        container_optional["path"] = param1_dict["path"]
    elif "multiSelector" in param1_dict.keys():
        container_optional["multiSelector"] = param1_dict["multiSelector"]
    if "timeout" in param1_dict.keys():
        container_optional["timeout"] = int(param1_dict["timeout"])
    else:
        container_optional["timeout"] = gr.get_frame_config_value(
            "wait_ele_timeout", 10
        )

    param3_dict = dsl_helper.params_to_dic(param3)

    search_dsl_str = param3_dict["selector"]
    search_optional = {}
    if "path" in param3_dict.keys():
        search_optional["path"] = param3_dict["path"]
    elif "multiSelector" in param3_dict.keys():
        search_optional["multiSelector"] = param3_dict["multiSelector"]

    swipe_count = None
    if "swipeCount" in param3_dict.keys():
        swipe_count = int(param3_dict["swipeCount"])
    else:
        swipe_count = gr.get_frame_config_value("swipe_search_count", 5)

    screen_size = gr.get_device_size()

    direction = param2.strip()

    start_x = None
    if "startX" in param3_dict.keys():
        start_x = float(param3_dict["startX"])
    start_y = None
    if "startY" in param3_dict.keys():
        start_y = float(param3_dict["startY"])

    distance = None
    if "distance" in param3_dict.keys():
        distance = float(param3_dict["distance"])
    else:
        distance = gr.get_frame_config_value("swipe_search_distance", 0.3)

    duration = None
    if gr.get_frame_config_value("use_search_swipe_duration", False):
        duration = gr.get_frame_config_value("search_swipe_duration", 1)
    if "duration" in param3_dict.keys():
        duration = float(param3_dict["duration"])

    ps.air_bdd_swipe_search(
        poco_instance,
        container_dsl_str,
        container_optional,
        search_dsl_str,
        search_optional,
        swipe_count,
        screen_size,
        direction,
        start_x,
        start_y,
        distance,
        duration,
    )


def handle_str(un_handle_str):
    res = re.match(r"([\S\s]+),\s*[0-9_]+\s*", un_handle_str)
    if res is not None:
        return res.group(1)
    else:
        return un_handle_str


def full_screen_swipe_to_ele_aaa(context, param1, param2):
    poco_instance = gr.get_value("pocoInstance")
    handled_param2_temp = handle_str(param2)
    param2_dict = dsl_helper.params_to_dic(handled_param2_temp)

    search_dsl_str = param2_dict["selector"]
    search_optional = {}
    if "path" in param2_dict.keys():
        search_optional["path"] = param2_dict["path"]
    elif "multiSelector" in param2_dict.keys():
        search_optional["multiSelector"] = param2_dict["multiSelector"]

    swipe_count = None
    if "swipeCount" in param2_dict.keys():
        swipe_count = int(param2_dict["swipeCount"])
    else:
        swipe_count = gr.get_frame_config_value("swipe_search_count", 5)

    direction = param1.strip()

    screen_size = gr.get_device_size()

    start_x = None
    if "startX" in param2_dict.keys():
        start_x = float(param2_dict["startX"])
    start_y = None
    if "startY" in param2_dict.keys():
        start_y = float(param2_dict["startY"])

    distance = None
    if "distance" in param2_dict.keys():
        distance = float(param2_dict["distance"])
    else:
        distance = gr.get_frame_config_value("swipe_search_distance", 0.3)

    duration = None
    if gr.get_frame_config_value("use_search_swipe_duration", False):
        duration = gr.get_frame_config_value("search_swipe_duration", 1)
    if "duration" in param2_dict.keys():
        duration = float(param2_dict["duration"])

    ps.full_screen_swipe_search(
        poco_instance,
        search_dsl_str,
        search_optional,
        swipe_count,
        direction,
        screen_size,
        start_x,
        start_y,
        distance,
        duration,
    )



# coding=utf-8
"""
behave hook and our plugin load from here
"""
from flybirds.core.global_context import GlobalContext
from flybirds.core.plugin.life_cycle import load
from flybirds.utils import flybirds_log as log

__import__("flybirds.core.plugin.life_cycle")


def before_all(context):
    """
    behave hook before running
    """
    # before start load
    log.info(f'[before_all_hook] user_data:{context.config.userdata}')
    load(context)
    # load plugins
    GlobalContext.process("config_processor", context)
    GlobalContext.process("before_run_processor", context)


def after_all(context):
    """
    behave global hook after running
    """
    GlobalContext.process("after_run_processor", context)


def before_feature(context, feature):
    """
    feature hook before running
    """
    GlobalContext.process("before_feature_processor", context, feature)


def after_feature(context, feature):
    """
    feature hook after running
    """
    GlobalContext.process("after_feature_processor", context, feature)


def before_scenario(context, scenario):
    """
    scene before running hook
    """
    GlobalContext.process("before_scenario_processor", context, scenario)


def after_scenario(context, scenario):
    """
    scenario post-run hook
    """
    GlobalContext.process("after_scenario_processor", context, scenario)


def before_step(context, step):
    """
    statement hook before run
    """
    GlobalContext.process("before_step_processor", context, step)


def after_step(context, step):
    """
    statement run hook
    """
    GlobalContext.process("after_step_processor", context, step)


def before_tag(context, tag):
    """
    tag front hook
    """
    GlobalContext.process("before_tag_processor", context, tag)


def after_tag(context, tag):
    """
    tag post hook
    """
    GlobalContext.process("after_tag_processor", context, tag)



# -*- coding: utf-8 -*-
"""
ui driver
"""
import flybirds.core.global_resource as gr
from flybirds.core.plugin.plugins.default.ui_driver.poco.poco_manage import \
    poco_init
from flybirds.core.plugin.plugins.default.ui_driver.paddleocr.ocr_manage import \
    ocr_init
from flybirds.core.plugin.plugins.default.ui_driver.poco.poco_screen import \
    air_bdd_screen_size

__open__ = ["UIDriver"]


class UIDriver:
    """
    ui driver
    """
    name = "ios_ui_driver"

    @staticmethod
    def init_driver():
        return poco_init()

    @staticmethod
    def init_ocr():
        return ocr_init()

    @staticmethod
    def air_bdd_screen_size(dr_instance):
        return air_bdd_screen_size(dr_instance)

    @staticmethod
    def close_driver():
        screen_record = gr.get_value("screenRecord")
        if screen_record is not None and hasattr(screen_record, "destroy"):
            screen_record.destroy()



# -*- coding: utf-8 -*-
globalization = {
    "en": {
        "rerun failed scenario": u"rerun failed scenario",
        "information association of failed operation":
            "information association of failed operation, run the {} time"
            " :[{}]",
        "start record": "start record",
        "stop record": "stop record",
        "rank": "rank",
        "parent": "parent",
        "children": "children",
        "sibling": "sibling",
        "offsprings": "offsprings",
        "and": "and",
        "up": "up",
        "down": "down",
        "left": "left",
        "right": "right",
        "modal_list": ["name=同意并继续",
                       "name=使用App时允许",
                       "name=允许",
                       "name=common update X",
                       "name=我知道了",
                       "text=同意并继续",
                       "text=使用App时允许",
                       "text=允许",
                       "text=common update X",
                       "text=我知道了",
                       "text=System UI isn't responding",
                       "text=Wait"]
    },
    "zh-CN": {
        "rerun failed scenario": u"\u5931\u8d25\u91cd\u65b0\u8fd0\u884c",
        "information association of failed operation":
            "失败运行的信息关联,运行第{}次:[{}]",
        "start record": u"开始录屏",
        "stop record": u"结束录屏",
        "rank": u"第",
        "parent": u"父节点",
        "children": u"孩子",
        "sibling": u"兄弟",
        "offsprings": u"后代",
        "and": u"并且",
        "up": u"上",
        "down": u"下",
        "left": u"左",
        "right": u"右",
        "modal_list": ["name=同意并继续",
                       "name=使用App时允许",
                       "name=允许",
                       "name=common update X",
                       "name=我知道了",
                       "text=同意并继续",
                       "text=使用App时允许",
                       "text=允许",
                       "text=common update X",
                       "text=我知道了",
                       "text=System UI isn't responding",
                       "text=Wait"]
    },
}

step_language = {
    "en": {
        "install app[{param}]": ["install app[{param}]"],
        "start app[{param}]": ["start app[{param}]"],
    },
    "zh-CN": {
        "install app[{selector}]": ["安装APP[{selector}]"],
        "delete app[{selector}]": ["删除APP[{selector}]"],
        "start app[{selector}]": ["启动APP[{selector}]"],
        "restart app": ["重启App"],
        "close app": ["关闭App"],
        "init device[{selector}]": ["设备初始化[{selector}]"],
        "connect device[{selector}]": ["连接设备[{selector}]"],
        "start recording timeout[{param}]": ["开始录屏超时[{param}]"],
        "start record": ["开始录屏"],
        "stop record": ["结束录屏"],
        "go to url[{param}]": ["跳转到[{param}]", "跳转页面到[{param}]"],
        "return to previous page": ["返回上一页"],
        "go to home page": ["回到首页"],
        "logon account[{selector1}]password[{selector2}]": [
            "登录账号[{selector1}]密码[{selector2}]",
            "登陆账号[{selector1}]密码[{selector2}]",
        ],
        "logout": ["退出登陆", "退出登录"],
        "wait[{param}]seconds": ["等待[{param}]秒"],
        "screenshot": ["全屏截图"],
        "ocr": ["全屏扫描"],
        "information association of failed operation, run the {param1} time"
        " :[{param2}]": ["失败运行的信息关联,运行第{param1}次:[{param2}]"],
        "text[{selector}]property[{param2}]is {param3}": [
            "文案[{selector}]的属性[{param2}]为{param3}"
        ],
        "element[{selector}]property[{param2}]is {param3}": [
            "元素[{selector}]的属性[{param2}]为{param3}"
        ],
        "click[{selector}]": ["点击[{selector}]"],
        "click text[{selector}]": ["点击文案[{selector}]"],
        "click ocr text[{selector}]": ["点击扫描文案[{selector}]"],
        "click position[{x},{y}]": ["点击屏幕位置[{x},{y}]"],
        "in[{selector}]input[{param2}]": ["在[{selector}]中输入[{param2}]"],
        "in ocr[{selector}]input[{param2}]": ["在扫描文字[{selector}]中输入[{param2}]"],
        "clear [{selector}] and input[{param2}]": [
            "在[{selector}]中清空并输入[{param2}]"],
        "element[{selector}]position not change in[{param2}]seconds": [
            "元素[{selector}]位置[{param2}]秒内未变动"
        ],
        "[{selector}]slide to {param2} distance[{param3}]": [
            "[{selector}]向{param2}滑动[{param3}]"],
        "slide to {param1} distance[{param2}]": ["全屏向{param1}滑动[{param2}]"],
        "exist text[{selector}]": ["存在[{selector}]的文案"],
        "ocr exist text[{selector}]": ["扫描存在[{selector}]的文案"],
        "not exist text[{selector}]": ["不存在[{selector}]的文案"],
        "ocr not exist text[{selector}]": ["扫描不存在[{selector}]的文案"],
        "text[{selector}]disappear": ["文案[{selector}]消失"],
        "exist[{selector}]element": ["存在[{selector}]的元素"],
        "not exist element[{selector}]": ["不存在[{selector}]的元素"],
        "element[{selector}]disappear": ["元素[{selector}]消失"],
        "the text of element[{selector}]is[{param2}]": [
            "[{selector}]的文案为[{param2}]"
        ],
        "the text of element[{selector}]include[{param2}]": [
            "[{selector}]的文案包含[{param2}]"
        ],
        "page rendering complete appears element[{selector}]": [
            "页面渲染完成出现元素[{selector}]"],
        "page ocr complete find text[{selector}]": [
            "页面扫描完成出现文字[{selector}]"],
        "existing element[{selector}]": ["存在元素[{selector}]"],

        "in[{p_selector}]from {param2} find[{c_selector}]element": [
            "在[{p_selector}]中向{param2}查找[{c_selector}]的元素"
        ],
        "from {param1} find[{selector}]element": [
            "向{param1}查找[{selector}]的元素"],

        "unblock the current page": ["解除当前页面限制"],
        "current page is [{param}]": ["当前页面是[{param}]"],
        "current page is not last page": ["当前页面已不是上一个指定页面"],
        "from [{selector}] select [{param2}]": ["在[{selector}]中选择[{param2}]"],
        "exist [{p_selector}] subNode [{c_selector}] element": [
            "存在[{p_selector}]的[{c_selector}]的元素"],
        "the text of element [{p_selector}] subNode [{c_selector}] is "
        "[{param3}]": [
            "[{p_selector}]的[{c_selector}]文案为[{param3}]"
        ],
        "cache service request [{service}]": ["缓存服务请求[{service}]"],
        "remove service request cache [{service}]": ["移除请求缓存[{service}]"],
        "remove all service request caches": ["移除所有请求缓存"],
        "listening service [{service}] bind mockCase[{mock_case_id}]": [
            "监听服务[{service}]绑定MockCase[{mock_case_id}]"],
        "remove service listener [{service}]": ["移除服务监听[{service}]"],
        "remove all service listeners": ["移除所有服务监听"],
        "compare service request [{service}] with json file "
        "[{target_data_path}]": [
            "验证服务请求[{service}]与[{target_data_path}]一致"],
        "compare service non-json request [{service}] with non-json file "
        "[{target_data_path}]": [
            "验证服务非json请求[{service}]与[{target_data_path}]一致"],
        "service request [{service}] request parameter [{target_json_path}] "
        "is [{expect_value}]": [
            "验证服务[{service}]的请求参数[{target_json_path}]"
            "与[{expect_value}]一致"]

    },
}



# -*- coding: utf-8 -*-
"""
file helper
"""
import json
import os
import re
from json import JSONDecodeError

from flybirds.utils import flybirds_log as log


def store_json_to_file_path(data, path, write_type):
    """
    json save to file
    """
    with open(path, write_type) as fw:
        json.dump(data, fw)


def get_json_from_file_path(path):
    """
    Get the content of the json file and convert it into an object
    """
    f = None
    result = None
    # noinspection PyBroadException
    try:
        f = open(path, "r", errors='ignore', encoding="utf-8")
        json_str = f.read().strip().replace('\r\n', '\\r\\n')
        result = json.loads(json_str, strict=False)
    except JSONDecodeError:
        log.warn('get_json_from_file_path has error.')
    finally:
        if f:
            f.close()
    return result


def get_json_from_file(file_name):
    """
    Get the content of the json file and convert it into an object
    """
    f = None
    result = None
    # noinspection PyBroadException
    try:
        f = open(file_name, "r", encoding="utf-8")
        result = json.load(f)
    except Exception as e:
        raise e
    finally:
        if f:
            f.close()
    return result


def create_dirs(path):
    """
    Check and create path
    """
    path = os.path.join(os.getcwd(), path)
    dir_exist = os.path.exists(path)
    if not dir_exist:
        os.makedirs(path)


def create_dirs_path_object(path):
    """
    Check and create path
    """
    dir_exist = os.path.exists(path)
    if not dir_exist:
        os.makedirs(path)
        return True
    else:
        return False


def clear_dirs(path):
    """
    Clear all files and folders under a folder
    """
    # cwd_pth = os.getcwd()
    # if os.environ.get('base_dir') is not None:
    #     cwd_pth = os.environ.get('base_dir')
    path = os.path.join(os.getcwd(), path)
    if os.path.exists(path):
        ls = os.listdir(path)
        for item in ls:
            c_path = os.path.join(path, item)
            if os.path.isdir(c_path):
                clear_dirs(c_path)
                os.rmdir(c_path)
            else:
                os.remove(c_path)


def valid_file_name(o_name):
    """
    Replace the illegal string in the file
    """
    in_valid_chars = "|\\?*<\":>+[]/',"
    for c in in_valid_chars:
        o_name = o_name.replace(c, "_")
    return o_name.replace(" ", "_")


def array_to_file(file_path, str_array):
    """
    Overwrite data in the file,
    if it does not exist, create a new file
    """
    if isinstance(str_array, list) and len(str_array) > 0:
        f = None
        # noinspection PyBroadException
        try:
            f = open(file_path, "w+", encoding="utf-8")
            for str_index in range(len(str_array)):
                f.write(str(str_array[str_index]))
        except Exception as e:
            raise e
        finally:
            if f:
                f.close()


def get_files_from_dir(file_path):
    """
    Recursively get all files from the directory
    """
    files = []
    for main_dir, dirs, file_name_list in os.walk(file_path):
        for file in file_name_list:
            file_path = os.path.join(main_dir, file)
            files.append(file_path)
    return files


def get_paths_from_dir(file_path, dir_name):
    """
    Recursively get all file paths from the directory
    """
    paths = []
    for main_dir, dirs, file_name_list in os.walk(file_path):
        if dir_name is not None and main_dir.find(dir_name) != -1:
            paths.append(main_dir)
    return paths


def replace_file_content(file_path, key, value):
    """
    replace file content
    """
    new_file = ""
    with open(file_path, "r", encoding="utf-8") as f:
        new_file = f.read()
    new_file = new_file.replace(f"%{{{{{key}}}}}", f"{value}")
    # write file
    with open(file_path, "w", encoding="utf-8") as f:
        f.write(new_file)


def update(filename, text):
    content = ""
    with open(filename, "r", encoding="utf-8") as f:
        content = f.read()
    content = content + '\n' + text
    # write file
    with open(filename, "w", encoding="utf-8") as f:
        f.write(content)


def update_json_data(json_path, key, value):
    with open(json_path, "r") as jsonFile:
        data = json.load(jsonFile)
    key_arr = key.split('.')
    data[key_arr[0]][key_arr[1]] = value

    with open(json_path, "w") as jsonFile:
        json.dump(data, jsonFile, indent=2, separators=(',', ': '))


def read_file_from_path(path):
    """
    Get the content of the file by path
    """
    with open(path, 'r') as f:
        data = f.read()
    return data


def read_json_data(root_dir_path):
    all_json_data = {}
    if not os.path.exists(root_dir_path):
        log.warn(f'[read_json_data] does not exists path:{root_dir_path}')
        return all_json_data
    for main_dir, dirs, file_name_list in os.walk(root_dir_path):
        for file in file_name_list:
            file_path = os.path.join(main_dir, file)
            if re.search(r"\.json", str(file_path)) is not None:
                json_data = get_json_from_file_path(file_path)
                for key, value in json_data.items():
                    all_json_data[key] = value
    return all_json_data



# -*- coding: utf-8 -*-
# @Time : 2022/3/7 19:18
# @Author : hyx
# @File : page.py
# @desc : web page implement
import json
import time
from urllib.parse import urlparse

import flybirds.core.global_resource as global_resource
import flybirds.core.global_resource as gr
import flybirds.utils.flybirds_log as log
import flybirds.utils.verify_helper as verify_helper
from flybirds.core.plugin.plugins.default.web.interception import \
    get_case_response_body
from flybirds.utils import dsl_helper
from flybirds.utils.dsl_helper import is_number

__open__ = ["Page"]


class Page:
    """Web Page Class"""

    name = "web_page"
    instantiation_timing = "plugin"

    def __init__(self):
        page, context = self.init_page()
        self.page = page
        self.context = context

    @staticmethod
    def init_page():
        context = Page.new_browser_context()

        page = context.new_page()
        request_interception = gr.get_web_info_value("request_interception",
                                                     True)
        if request_interception:
            page.route("**/*", handle_route)
            # request listening events
            page.on("request", handle_request)

        ele_wait_time = gr.get_frame_config_value("wait_ele_timeout", 30)
        page_render_timeout = gr.get_frame_config_value("page_render_timeout",
                                                        30)
        page.set_default_timeout(float(ele_wait_time) * 1000)
        page.set_default_navigation_timeout(float(page_render_timeout) * 1000)
        return page, context

    @staticmethod
    def new_browser_context():
        browser = gr.get_value('browser')

        operation_module = gr.get_value("projectScript").custom_operation
        create_browser_context = getattr(operation_module,
                                         "create_browser_context")
        context = create_browser_context(browser)
        if context is not None:
            log.info('[new_browser_context] successfully get BrowserContext '
                     'from custom operation')
            return context
        context = browser.new_context(record_video_dir="videos",
                                      ignore_https_errors=True)
        return context

    def navigate(self, context, param):
        operation_module = gr.get_value("projectScript").custom_operation
        page_url = None
        if hasattr(operation_module, "get_page_url"):
            get_page_url = getattr(operation_module,
                                   "get_page_url")
            page_url = get_page_url(param)

        if page_url is not None:
            log.info('[get_page_url] successfully get page_url_value '
                     'from custom operation')
            self.page.goto(page_url)
            return

        param_dict = dsl_helper.params_to_dic(param, "urlKey")
        url_key = param_dict["urlKey"]
        schema_url_value = gr.get_page_schema_url(url_key)
        self.page.goto(schema_url_value)

    def return_pre_page(self, context):
        self.page.go_back()

    def sleep(self, context, param):
        if is_number(param):
            self.page.wait_for_timeout(float(param) * 1000)
        else:
            log.warn("default wait for timeout!")
            self.page.wait_for_timeout(3 * 1000)

    def cur_page_equal(self, context, param):
        cur_url = self.page.url.split('?')[0]
        if param.startswith(("http", "https")):
            target_url = param.split('?')[0]
        else:
            schema_url = global_resource.get_page_schema_url(param)
            target_url = schema_url
        verify_helper.text_equal(target_url, cur_url)


def handle_request(request):
    # interception request handle
    parsed_uri = urlparse(request.url)
    operation = parsed_uri.path.split('/')[-1]
    if operation is not None:
        interception_request = gr.get_value('interceptionRequest')
        request_body = interception_request.get(operation)

        if request_body is not None:
            log.info(
                f'[handle_request] start cache service：{operation}')
            current_request_info = {'postData': request.post_data,
                                    'url': request.url,
                                    'updateTimeStamp': int(
                                        round(time.time() * 1000))}
            interception_request[operation] = current_request_info
            gr.set_value("interceptionRequest", interception_request)


def handle_route(route):
    abort_domain_list = gr.get_web_info_value("abort_domain_list", [])
    parsed_uri = urlparse(route.request.url)
    domain = parsed_uri.hostname
    if abort_domain_list and domain in abort_domain_list:
        route.abort()
        return

    resource_type = route.request.resource_type
    if resource_type != 'fetch' and resource_type != 'xhr':
        route.continue_()
        return

    # mock response data
    operation = parsed_uri.path.split('/')[-1]
    mock_case_id = None
    if operation is not None:
        interception_values = gr.get_value('interceptionValues')
        mock_case_id = interception_values.get(operation)
    if mock_case_id:
        mock_body = get_case_response_body(mock_case_id)
        if mock_body:
            if not isinstance(mock_body, str):
                mock_body = json.dumps(mock_body)
            route.fulfill(status=200,
                          content_type="application/json;charset=utf-8",
                          body=mock_body)
    else:
        route.continue_()



# -*- coding: utf-8 -*-
"""
report gen
"""
import os
import subprocess

from flybirds.report.gen_factory import GenFactory
from flybirds.utils.flybirds_log import logger

STATIC_DIR = os.path.dirname(__file__)


class CucumberGen:
    """
    cucumber gen
    """
    name = "cucumber"

    @staticmethod
    def gen(report_path, platform):
        """
        report gen
        """

        report_js_path = os.path.dirname(STATIC_DIR)

        gen_path = os.path.join(report_js_path, "node_report",
                                "report.js")

        cmd_str = f"node {gen_path} {report_path} {report_path} {platform}"
        logger.info(f"report cmd:{cmd_str}")

        report_process = subprocess.Popen(
            cmd_str, cwd=os.getcwd(), shell=True, stdout=None
        )

        report_process.wait()
        report_process.communicate()


GenFactory.add(CucumberGen)



# -*- coding: utf-8 -*-
"""
project cli
"""
from typing import List, Optional

import typer

import flybirds.utils.flybirds_log as log
from flybirds.cli.create_project import create_demo
from flybirds.cli.parse_args import parse_args, default_report_path
from flybirds.core.launch_cycle.run_manage import run_script

app = typer.Typer(
    help='Welcome to flybirds. Type "--help" for more information.',
    no_args_is_help=True,
)


@app.command("run")
def runner(
        feature_path: str = typer.Option(
            "features",
            "--path",
            "-P",
            help="Feature path that needs " "to be executed",
        ),
        tag: str = typer.Option(
            None,
            "-T",
            "--tag",
            help="Run scenarios with a specific tag. "
                 "Multiple scenarios are separated by "
                 "commas(,). "
                 "e.g. flybirds run --tag tag1,-tag2,tag4",
        ),
        report_format: str = typer.Option(
            "--format=json", "--format", "-F", help="Result format."
        ),
        report_path: str = typer.Option(
            default_report_path,
            "-R",
            "--report",
            help="The path to generate the report.",
        ),
        define: Optional[List[str]] = typer.Option(
            None,
            "-D",
            "--define",
            help="User-defined parameters. e.g. --define headless=false.",
        ),
        rerun: bool = typer.Option(
            True,
            "--rerun/--no-rerun",
            help="Whether the failed scenario needs to be rerun",
        ),
        es: str = typer.Option(
            None, "--es", help="APP boot environment parameters"
        ),
        to_html: bool = typer.Option(
            True, "--html/--no-html", help="Whether to generate HTML report"
        ),
        run_at: str = typer.Option(
            "local", "--run-at", help="Run environment, extended parameters"
        ),
        processes: int = typer.Option(
            4, "--processes", '-p',
            help="Maximum number of processes. Default = 4. Effective when  "
                 "test on web."
        )
):
    """
    Run the project.
    """
    # process args
    run_args = parse_args(
        feature_path, tag, report_format, report_path, define, rerun, es,
        to_html, run_at, processes
    )
    log.info("============last run_args: {}".format(str(run_args)))
    run_script(run_args)


@app.command("create")
def create_project():
    """
    Generate project example
    """
    create_demo()


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
"""
web Element core api implement
"""

__open__ = ["Element"]

import flybirds.core.global_resource as gr
import flybirds.utils.flybirds_log as log
import flybirds.utils.verify_helper as verify_helper
from flybirds.core.exceptions import FlybirdVerifyException, \
    FlybirdsVerifyEleException
from flybirds.core.global_context import GlobalContext as g_Context
from flybirds.utils import language_helper as lan
from flybirds.utils.dsl_helper import handle_str, params_to_dic


def direct_left(x, y, diff):
    to_x = x - diff
    to_y = y
    return to_x, to_y


def direct_right(x, y, diff):
    to_x = x + diff
    to_y = y
    return to_x, to_y


def direct_up(x, y, diff):
    to_x = x
    to_y = y - diff
    return to_x, to_y


def direct_down(x, y, diff):
    to_x = x
    to_y = y + diff
    return to_x, to_y


def direct_default(x, y, diff):
    to_x = x
    to_y = y - diff if y - diff > 0 else 0
    return to_x, to_y


direct_dict = {
    'left': direct_left,
    'right': direct_right,
    'up': direct_up,
    'down': direct_down
}


class Element:
    """Web Element Class"""

    name = "web_element"
    instantiation_timing = "plugin"

    def __init__(self):
        page_obj = gr.get_value("plugin_page")
        if page_obj is None or (not hasattr(page_obj, 'page')):
            log.error('[web Element init] get page object has error!')
        self.page = page_obj.page

    def get_ele_locator(self, selector):
        if selector is None:
            message = f"[get_ele_locator] the param[{selector}] is None."
            raise FlybirdsVerifyEleException(message=message)
        param_temp = handle_str(selector)
        param_dict = params_to_dic(param_temp)
        selector_str = param_dict["selector"]

        if "timeout" in param_dict.keys():
            timeout = param_dict["timeout"]
        else:
            timeout = gr.get_frame_config_value("wait_ele_timeout", 30)

        ele_locator = self.page.locator(selector_str)
        return ele_locator, float(timeout) * 1000

    def get_ele_text(self, param):
        locator, timeout = self.get_ele_locator(param)
        e_text = locator.inner_text(timeout=timeout)
        if e_text is None or e_text.strip() == '':
            e_text = locator.get_attribute('value')
            if e_text is None or e_text.strip() == '':
                e_text = ""
        return e_text

    def ele_click(self, context, param):
        locator, timeout = self.get_ele_locator(param)
        locator.click(timeout=timeout)

    def click_text(self, context, param):
        if 'text=' not in param:
            param = "text=" + param
        locator, timeout = self.get_ele_locator(param)
        locator.click(timeout=timeout)

    def click_coordinates(self, context, x, y):
        self.page.mouse.click(float(x), float(y))

    def ele_text_include(self, context, param_1, param_2):
        e_text = self.get_ele_text(param_1)
        verify_helper.text_container(param_2, e_text)

    def find_text(self, context, param):
        param_temp = handle_str(param)
        param_dict = params_to_dic(param_temp)
        selector_str = param_dict["selector"]

        p_content = self.page.content()
        if selector_str in p_content:
            log.info(f'find_text: [{selector_str}] is success!')
        else:
            message = f"expect to find the text [{selector_str}] in the " \
                      f"page, but not actually find it"
            raise FlybirdVerifyException(message)

    def find_no_text(self, context, param):
        param_temp = handle_str(param)
        param_dict = params_to_dic(param_temp)
        selector_str = param_dict["selector"]

        p_content = self.page.content()
        if selector_str in p_content:
            message = f"except text [{selector_str}] not exists in page, " \
                      f"but actual has find it."
            raise FlybirdVerifyException(message)

    def ele_text_equal(self, context, param_1, param_2):
        e_text = self.get_ele_text(param_1)
        verify_helper.text_equal(param_2, e_text)

    def ele_exist(self, context, param):
        locator, timeout = self.get_ele_locator(param)
        locator.element_handle(timeout=timeout)

    def ele_not_exist(self, context, param):
        try:
            self.ele_exist(context, param)
            ele_exists = True
        except Exception:
            ele_exists = False

        if ele_exists:
            message = f"except element [{param}] not exists in page, " \
                      f"but actual has find it."
            raise FlybirdVerifyException(message)

    def wait_for_ele(self, context, param):
        locator, timeout = self.get_ele_locator(param)
        locator.wait_for(timeout=timeout, state='visible')

    def ele_input_text(self, context, param_1, param_2):
        locator, timeout = self.get_ele_locator(param_1)
        locator.click(timeout=timeout)
        locator.fill(param_2, timeout=timeout)
        return self.page.wait_for_timeout(100)

    def clear_and_input(self, context, param_1, param_2):
        locator, timeout = self.get_ele_locator(param_1)
        locator.click(timeout=timeout)
        locator.fill('', timeout=timeout)
        locator.fill(param_2, timeout=timeout)
        return self.page.wait_for_timeout(100)

    def ele_slide(self, context, param_1, param_2, param_3):
        locator, timeout = self.get_ele_locator(param_1)
        box = locator.bounding_box(timeout=timeout)
        x = box["x"] + box["width"] / 2
        y = box["y"] + box["height"] / 2
        # get scroll direction
        language = g_Context.get_current_language()
        direct = lan.get_glb_key(param_2, language)

        fun = direct_dict.get(direct, direct_default)
        to_x, to_y = fun(x, y, float(param_3))

        self.page.evaluate(f"window.scrollTo({to_x}, {to_y})")

    def full_screen_slide(self, context, param_1, param_2):
        # get scroll direction
        language = g_Context.get_current_language()
        direct = lan.get_glb_key(param_1, language)

        fun = direct_dict.get(direct, direct_default)
        to_x, to_y = fun(0, 0, float(param_2))

        self.page.evaluate(f"window.scrollBy({to_x}, {to_y})")

    def ele_select(self, context, selector, option_str):
        locator, timeout = self.get_ele_locator(selector)
        try:
            # select by text
            locator.select_option(label=option_str, timeout=timeout)
            log.info(f'[ele_select] select option[{option_str}] success.')
        except FlybirdsVerifyEleException as fe:
            raise fe
        except Exception:
            # select by value
            log.warn(f'[ele_select] retry select option[{option_str}].')
            locator.select_option(option_str, timeout=timeout)

    def find_full_screen_slide(self, context, param1, param2):
        locator, timeout = self.get_ele_locator(param2)
        locator.scroll_into_view_if_needed(timeout=timeout)

    def get_ele_attr(self, selector, attr_name, params_deal_module=None,
                     deal_method=None):
        locator, timeout = self.get_ele_locator(selector)
        ele_attr = locator.get_attribute(attr_name, timeout=timeout)
        if deal_method is not None:
            deal_method = getattr(params_deal_module, deal_method)
            ele_attr = deal_method(ele_attr)
        return ele_attr

    def is_ele_attr_equal(self, context, selector, attr_name, target_val):
        param2_dict = params_to_dic(attr_name, "attrName")
        target_attr = param2_dict["attrName"]

        deal_method = None
        params_deal_module = None
        if "dealMethod" in param2_dict.keys():
            deal_method = param2_dict["dealMethod"]
            params_deal_module = gr.get_value("projectScript").params_deal

        ele_attr = self.get_ele_attr(selector, target_attr, params_deal_module,
                                     deal_method)
        verify_helper.attr_equal(target_val, ele_attr)

    def is_text_attr_equal(self, context, text_selector, attr_name,
                           target_val):
        if 'text=' not in text_selector:
            text_selector = "text=" + text_selector
        self.is_ele_attr_equal(context, text_selector, attr_name, target_val)

    def is_parent_exist_child(self, context, parent_selector, child_selector):
        parent_locator, p_timeout = self.get_ele_locator(parent_selector)

        child_temp = handle_str(child_selector)
        child_dict = params_to_dic(child_temp)
        child_str = child_dict["selector"]

        if "timeout" in child_dict.keys():
            c_timeout = child_dict["timeout"]
        else:
            c_timeout = gr.get_frame_config_value("wait_ele_timeout", 30)
        c_timeout = float(c_timeout) * 1000

        sub_locator = parent_locator.locator(child_str)
        sub_locator.element_handle(timeout=p_timeout)
        return sub_locator, c_timeout

    def find_text_from_parent(self, context, parent_selector, child_selector,
                              target_text):
        sub_locator, c_timeout = self.is_parent_exist_child(context,
                                                            parent_selector,
                                                            child_selector)
        e_text = sub_locator.inner_text(timeout=c_timeout)
        if e_text is None or e_text.strip() == '':
            e_text = sub_locator.get_attribute('value', timeout=c_timeout)
        verify_helper.text_equal(target_text, e_text)



# -*- coding: utf-8 -*-
"""
http help
"""
import requests


def http_get(url, param=None, header=None):
    if url is not None and url != "":
        result = requests.get(url, params=param, headers=header, )
        j_obj = result.json()
        result.close()
        return j_obj
    else:
        return None



"""
android app core api implement
"""

from airtest.core.api import (time, start_app, stop_app, install, uninstall)

__open__ = ["App"]


class App:
    """Android App Class"""

    name = "android_app"

    def wake_app(self, package_name, wait_time=None):
        """
        Start the target application on device
        """
        start_app(package_name)
        if not (wait_time is None):
            time.sleep(wait_time)

    def shut_app(self, package_name):
        """
        关闭测试app
        """
        stop_app(package_name)

    def install_app(self, package_path, wait_time=None):
        """
        Install application on device

        :param package_path: the path to file to be installed on target device
        :param wait_time:
        :return: None
        :platforms: Android
        :Example:
            >>> install_app(r"D:\\demo\\test.apk")
        """
        i_result = install(package_path)
        if not (wait_time is None):
            time.sleep(wait_time)
        return i_result

    def uninstall_app(self, package_name, wait_time=None):
        """
        Uninstall application on device

        :param package_name: name of the package, see also `start_app`
        :param wait_time:
        :return: None
        :platforms: Android
        :Example:
            >>> uninstall("com.flyBirds.music")
        """
        uninstall(package_name)
        if not (wait_time is None):
            time.sleep(wait_time)



# -*- coding: utf-8 -*-
# @Time : 2022/2/24 19:27
# @Author : hyx
# @File : web_run.py
# @desc : web_run
import traceback

import flybirds.core.global_resource as gr
import flybirds.utils.flybirds_log as log
from flybirds.core.driver import ui_driver
from flybirds.core.global_context import GlobalContext
from flybirds.utils import launch_helper


class OnBefore:
    name = "OnWebBefore"
    order = 51

    @staticmethod
    def can(context):
        if gr.get_platform() is not None \
                and (gr.get_platform().lower() == "web"):
            return True
        else:
            return False

    @staticmethod
    def init_ui_driver(context):
        """
        init browser
        """
        play_wright, browser = ui_driver.init_driver()
        context.play_wright = play_wright
        context.browser = browser
        GlobalContext.ui_driver_instance = play_wright
        log.info("playwright object initialization completed")

    @staticmethod
    def init_ui_page(context):
        # get the global object used to plugin page
        plugin_page = GlobalContext.page()
        gr.set_value("plugin_page", plugin_page)
        context.plugin_page = plugin_page

        # get the global object used to record the screen
        screen_record = GlobalContext.screen_record()
        gr.set_value("screenRecord", screen_record)
        context.screen_record = screen_record

        plugin_ele = GlobalContext.element()
        gr.set_value("plugin_ele", plugin_ele)
        context.plugin_ele = plugin_ele

        log.info("screen recording context initialization completed")
        if not screen_record.support:
            log.info("the device does not support screen recording")

    @staticmethod
    def run(context):
        try:
            log.info('[web run] OnBefore run event!')
            OnBefore.init_ui_driver(context)
            OnBefore.init_ui_page(context)

        except Exception as init_error:
            log.info("global initialization error", traceback.format_exc())
            raise init_error
        # hook extend by tester
        before_all_extend = launch_helper.get_hook_file("before_all_extend")
        if before_all_extend is not None:
            before_all_extend(context)


class OnAfter:
    """
    after event
    """

    name = "OnWebAfter"
    order = 100

    @staticmethod
    def can(context):
        if gr.get_platform() is not None \
                and (gr.get_platform().lower() == "web"):
            return True
        else:
            return False

    @staticmethod
    def run(context):
        log.info('[web run] OnAfter run event!')
        # close browser
        ui_driver.close_driver()

        # hook extend by tester
        after_all_extend = launch_helper.get_hook_file("after_all_extend")
        if after_all_extend is not None:
            after_all_extend(context)


hook1 = GlobalContext.join("before_run_processor", OnBefore, 1)
hook2 = GlobalContext.join("after_run_processor", OnAfter, 1)



# -*- coding: utf-8 -*-
"""
parse json report
"""
import os
import re
import shutil
import traceback
from json import JSONDecodeError

from flybirds.utils import file_helper
from flybirds.utils import flybirds_log as log


def parse_json_data(report_dir, rerun_report_dir=None, is_parallel=False):
    """
    Parse the screenshot address in the behave json report,
    and use it when converting
    the follow-up cucumber json report into an html report
    """
    rerun_features = None
    not_aggregation = False

    if rerun_report_dir is not None:
        # get the array of all rerun json under the rerun_report_dir
        rerun_features = get_rerun_feature(rerun_report_dir, is_parallel)
        log.info(
            "parse_json_data move_rerun_screen, report_dir_path: "
            f"{report_dir}, rerun_report_dir_path: {rerun_report_dir}"
        )
        # move_rerun_screen(report_dir, rerun_report_dir)
        # copy_rerun_screen(report_dir, rerun_report_dir)

    if isinstance(rerun_features, list) and len(rerun_features) > 0:
        not_aggregation = True

    for file_item in os.listdir(report_dir):
        if re.search(r"\.json", str(file_item)) is not None:
            # noinspection PyBroadException
            try:
                file_path = os.path.join(report_dir, file_item)
                report_json = file_helper.get_json_from_file_path(file_path)
                if isinstance(report_json, list):
                    cur_json = []
                    cur_features = []
                    if not_aggregation:
                        report_json.extend(rerun_features)
                        not_aggregation = False
                    for feature in report_json:
                        parse_feature(feature, rerun_report_dir)
                        if (
                                isinstance(feature["elements"], list)
                                and len(feature["elements"]) > 0
                        ):
                            if is_parallel and feature.get('metadata') is None:
                                browser_name = file_item.split('.')[1]
                                feature["metadata"] = [
                                    {"name": "Browser",
                                     "value": browser_name
                                     }
                                ]
                            cur_features.append(feature)
                    cur_json.extend(cur_features)

                    file_helper.store_json_to_file_path(
                        cur_json, file_path, "w"
                    )
            except JSONDecodeError:
                log.warn('[parse_json_data] has error: Invalid json.')
            except Exception:
                log.warn(
                    f"error processing image address in {file_item}",
                    traceback.format_exc(),
                )


def parse_feature(feature, rerun_report_dir):
    """
    parse feature: exclude the data with status=rerun
    """
    if isinstance(feature["elements"], list):
        cur_scenarios = []
        for scenario in feature["elements"]:
            if scenario["type"] == "background":
                continue
            if scenario["status"] == "rerun":
                if rerun_report_dir is None:
                    scenario["status"] = "failed"
                else:
                    continue
            if isinstance(scenario["description"], list):
                desc_new_array = []
                for desc_item in scenario["description"]:
                    if desc_item is None:
                        continue
                    desc_item = desc_item.strip()
                    if desc_item == "initialization description_":
                        continue
                    elif desc_item.startswith("embeddingsTags, stepIndex="):
                        split_array = desc_item.split(",")
                        step_index = int(
                            split_array[1].strip().split("=", 1)[1].strip()
                        )
                        data = split_array[2].strip()
                        if (
                                isinstance(scenario["steps"], list)
                                and len(scenario["steps"]) > step_index
                        ):
                            images_dict = {
                                "mime_type": "text/html",
                                "data": data,
                            }
                            if (
                                    "embeddings"
                                    in scenario["steps"][step_index].keys()
                            ):
                                scenario["steps"][step_index][
                                    "embeddings"
                                ].append(images_dict)
                            else:
                                scenario["steps"][step_index]["embeddings"] = [
                                    images_dict
                                ]
                    else:
                        desc_new_array.append(desc_item)
                scenario["description"] = desc_new_array

            cur_scenarios.append(scenario)
        feature["elements"] = cur_scenarios


def get_rerun_feature(report_dir, is_parallel):
    """
    Get all the results of rerun after failure
    """
    if report_dir is None:
        return None
    result = []
    try:
        for file_item in os.listdir(report_dir):
            if re.search(r"\.json", str(file_item)) is not None:
                # noinspection PyBroadException
                try:
                    file_path = os.path.join(report_dir, file_item)
                    report_json = file_helper.get_json_from_file_path(
                        file_path
                    )
                    if isinstance(report_json, list):
                        for feature in report_json:
                            if is_parallel and feature.get('metadata') is None:
                                browser_name = file_item.split('.')[1]
                                feature["metadata"] = [
                                    {"name": "Browser",
                                     "value": browser_name
                                     }
                                ]
                    if isinstance(report_json, list) and len(report_json) > 0:
                        result.extend(report_json)
                except Exception:
                    log.warn(
                        f"summarize a single rerun feature: {file_item} error",
                        traceback.format_exc(),
                    )
    except Exception as e:
        log.warn(
            "An error occurred when re-run "
            f"the feature result: {str(e)}",
            traceback.format_exc(),
        )
    return result


def move_rerun_screen(report_dir, rerun_report_dir=None):
    """
    Some screenshots and other resources that failed after re-running
     are moved to the resources generated during
    the first formal operation, which is
    conducive to the integration of reports
    """
    try:
        if rerun_report_dir is None:
            return
        origin_screen_dir = os.path.join(report_dir, "screenshot")
        rerun_screen_dir = os.path.join(rerun_report_dir, "screenshot")
        if os.path.exists(rerun_screen_dir):
            if not os.path.exists(origin_screen_dir):
                file_helper.create_dirs(origin_screen_dir)
            for dir_item in os.listdir(rerun_screen_dir):
                shutil.move(
                    os.path.join(rerun_screen_dir, dir_item), origin_screen_dir
                )
    except Exception as e:
        log.warn(
            "An error occurred when copying the failed screenshot information"
            f" to the report screenshot directory: {str(e)}",
            traceback.format_exc(),
        )


def copy_rerun_screen(report_dir, rerun_report_dir=None):
    """
    copy rerun screen
    """
    origin_screen_dir = os.path.join(report_dir, "screenshot")
    rerun_screen_dir = os.path.join(rerun_report_dir, "screenshot")
    copy_file(rerun_screen_dir, origin_screen_dir)


def copy_file(source_dir, target_dir):
    """
    copy dir
    """
    for s_f in os.listdir(source_dir):
        source_f = os.path.join(source_dir, s_f)
        print("sourceF", source_f)
        target_f = os.path.join(target_dir, s_f)
        if os.path.isfile(source_f):
            shutil.copy(source_f, target_f)
            log.info(
                "Copy the failed screenshot to the report screenshot"
                f" directory: {source_f}"
            )
        if os.path.isdir(source_f):
            if os.path.exists(target_f):
                copy_file(source_f, target_f)
            else:
                os.mkdir(target_f)
                copy_file(source_f, target_f)



# -*- coding: utf-8 -*-
"""
This module defines the steps related to the device.
"""
from behave import step

from flybirds.core.global_context import GlobalContext as g_Context
from flybirds.utils.dsl_helper import get_params, ele_wrap


@step("init device[{selector}]")
@ele_wrap
def init_device(context, selector=None):
    g_Context.step.init_device(context, selector)


@step("connect device[{selector}]")
@ele_wrap
def connect_device(context, selector=None):
    g_Context.step.connect_device(context, selector)


@step("start recording timeout[{param}]")
def start_screen_record_timeout(context, param=None):
    (param_1,) = get_params(context, (param, "param"))
    g_Context.step.start_screen_record_timeout(context, param_1)


@step("start record")
def start_screen_record(context):
    g_Context.step.start_screen_record(context)


@step("stop record")
def stop_screen_record(context):
    g_Context.step.stop_screen_record(context)



# -*- coding: utf-8 -*-
"""
Poco manage api
"""
import flybirds.core.plugin.plugins.default.ui_driver.poco.parse_path as parse
import flybirds.core.plugin.plugins.default.ui_driver.poco.parse_selector \
    as msd
import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_selector as pc
from flybirds.core.global_context import GlobalContext as g_context


def poco_init():
    """
    Get the poco object used by the initialization global
    """
    return g_context.element.ui_driver_init()


def create_poco_object_by_dsl(poco, select_dsl_str, optional):
    """
    get the poco element of optional parameter structure and selector structure
    :param poco:
    :param select_dsl_str:
    :param optional:
    :return:
    """
    poco_object = None
    if optional is not None and "path" in optional.keys() \
            and optional["path"] == "true":
        poco_object = parse.create_path_poco(poco, select_dsl_str)
    elif (
            optional is not None and
            "multiSelector" in optional.keys()
            and optional["multiSelector"] == "true"
    ):
        poco_object = pc.create_poco_object(
            poco, msd.create_multi_selector(select_dsl_str)
        )
    else:
        poco_object = pc.create_poco_object(
            poco, msd.create_single_selector(select_dsl_str)
        )
    return poco_object



# -*- coding: utf-8 -*-
"""
ui driver proxy
"""
from flybirds.core.global_context import GlobalContext


def air_bdd_screen_size(dr_instance):
    return GlobalContext.ui_driver.air_bdd_screen_size(dr_instance)


def init_driver():
    return GlobalContext.ui_driver.init_driver()


def init_ocr():
    return GlobalContext.ui_driver.init_ocr()


def close_driver():
    return GlobalContext.ui_driver.close_driver()



# -*- coding: utf-8 -*-
"""
This module is used to define APP unique behavior
"""


def schema_deal_rule(page_name, schema_url):
    """
    Custom android schema jump rules

    > NOTE:  The schema jump rules within each app are inconsistent.

    The following is a simple example of a schema jump rule:

    assumptions:
     1.The jump rule inside the app is：
          schema_url = "trip://myUrl?url={}+type={}".format(schema_url, schema_type)
     2. The schema_url item has been configured in schema_url.json,
      such as: {"Home": "https://www.ctrip.com/"}

     ```py

         schema_type = 1

        # special processing according to keywords in schema_url
        keyword = 'rn_flight'
        if keyword in schema_url:
            schema_type = 1
            # Add your logic processing, for example： concatenate a pkg_id
             parameter to schema_url
            pkg_id = 123456
            schema_url = schema_url + "&pkgBuildId=" + str(pkg_id)
        # Handle schema_url of 'http://' or 'https'
        elif ('http://' in schema_url) or 'https' in schema_url:
            schema_type = 2
        elif '/index.html' in schema_url:
            schema_type = 3
        print('current schema:{}'.format(schema_url))
        schema_url = base64.b64encode(schema_url.encode('utf-8')).decode('utf-8')
        # the address that the app needs to jump to in the end
        schema_url = "trip://myUrl?url={}+type={}".format(schema_url, schema_type)
        return schema_url

    ```
    Parameters
        ----------
        page_name :
            The key of the schema_url to be jumped to
        schema_url : The value of the schema_url to be jumped to
    Returns
        -------
        schema_url string
    """
    return schema_url


def to_home():
    pass


def login(user_name, password):
    pass


def logout():
    pass


def get_mock_case_body(mock_case_id):
    """
    custom get mockCase response body

    :param mock_case_id: unique key for get mock data
    """
    mock_case_body = None
    # here add something to get mockCase response body...
    return mock_case_body


def create_browser_context(browser):
    """
    custom creates a new browser context.

    For related api, also see: https://playwright.dev/python/docs/api/class-browser#browser-new-context

    :param browser: the browser instance
    """
    context = None
    # here add something to create BrowserContext...
    #
    # For example, adding parameters when create,
    #   locale: language, viewport: screen size
    #
    # context = browser.new_context(record_video_dir="videos",
    #                               ignore_https_errors=True,
    #                               locale="en",
    #                               viewport={"width": 800, "height": 800})
    return context


def get_page_url(param):
    """
    Custom web get page url rules

    ```py
    # example 1
    import os
    from flybirds.utils import file_helper

    path = os.path.join(os.getcwd(), "config", "schema_url.json")
    if os.path.exists(path):
        c_f = file_helper.get_json_from_file(path)
        if c_f.__contains__(param):
                return c_f.get(param)

    # example 2
    url = "https://www.ctrip.com/"
    return url
    ```
    Parameters
        ----------
        param :
            The key of the schema_url to be jumped to in schema_url.config
    Returns
        -------
        url string
    """
    pass




# -*- coding: utf-8 -*-
"""
this class just a flag that help to find plugin
"""


class PluginProxy:  # pylint: disable=too-few-public-methods
    """
    just a flag
    """

    def __init__(self):
        pass



# -*- coding: utf-8 -*-
"""
ios screen imp
"""

__open__ = ["Screen"]

from flybirds.core.plugin.plugins.default.screen import BaseScreen


class Screen(BaseScreen):
    """
    screen imp
    """
    name = "ios_screen"

    # def screen_shot(self, path):
    #     screen.screen_shot(path)
    #
    # def screen_link_to_behave(self, scenario, step_index, tag=None):
    #     screen.screen_link_to_behave(scenario, step_index, tag)



# -*- coding: utf-8 -*-
"""
Custom Step statement
"""
from behave import step


@step("Create new milestone[{param1}]and[{param2}]")
def create_milestone_with_title(context, param1, param2):
    param1 = param1.strip()
    param2 = param2.strip()
    print(f'I create milestone with title {param1} and {param2}!')


@step("Close existing milestone[{param1}]and[{param2}]")
def delete_milestone_with_title(context, param1, param2):
    param1 = param1.strip()
    param2 = param2.strip()
    print(f'I delete milestone with title {param1} and{param2}!')



# -*- coding: utf-8 -*-
"""
Screen recording related.
"""
import datetime
import os
import time

from airtest.core.android.adb import ADB

import flybirds.core.global_resource as gr
import flybirds.utils.file_helper as file_helper
import flybirds.utils.flybirds_log as log
import flybirds.utils.snippet as cmd_helper
import flybirds.utils.uuid_helper as uuid_helper
from flybirds.core.exceptions import ScreenRecordException

airtest_adb_path = ADB.builtin_adb_path()


class ScreenRecord:
    def __init__(self):
        self.support = False
        self.start_time = None
        self.end_time = None
        self.process = None
        # 0 Just created 1 Reset state 2 Start recording
        self.status = 0

        self.record_support()
        self.use_airtest_record = gr.get_frame_config_value(
            "use_airtest_record", False
        )
        log.info("use_airtest_record: {}".format(self.use_airtest_record))

        self.dev = gr.get_value("deviceInstance")
        if self.use_airtest_record:
            # Use airtest to record screen
            self.recording_file = "/sdcard/test.mp4"
            self.output_file = "screen.mp4"
        else:
            self.recording_file = "/sdcard/flybirds.mp4"

    def reset(self):
        """
        Reset object
        """
        self.start_time = None
        self.end_time = None
        self.status = 1
        if not (self.process is None):
            proc_code = self.process.poll()
            if proc_code is None:
                self.process.terminate()
                time.sleep(2)
        self.clear_record()

    def destroy(self):
        """
        Destroy all recorded video processes after the task runs
        """
        if not (self.process is None):
            proc_code = self.process.poll()
            if proc_code is None:
                self.process.terminate()
                time.sleep(2)
        self.clear_record()

    def record_support(self):
        device_id = gr.get_device_id()

        cmd = "{} -s {} shell screenrecord --help".format(airtest_adb_path,
                                                          device_id)
        proc = cmd_helper.create_sub_process(cmd)
        proc.communicate(timeout=10)
        proc_code = proc.poll()

        if int(proc_code) == 0:
            self.support = True
            log.info(
                "record_support message:  proc_code:{}, support: {}".format(
                    str(proc_code), self.support
                )
            )
        proc.terminate()

    def start_record(self, timeout, bit_rate_level=1, bit_rate=None):
        """
        Start recording
        """
        log.info("start_record")
        log.info("bit_rate_level: {}".format(bit_rate_level))
        if bit_rate_level > 5:
            bit_rate_level = 5
        if self.use_airtest_record:
            if not bit_rate:
                screen_size = gr.get_device_size()
                bit_rate = screen_size[0] * screen_size[1] * bit_rate_level
            max_time = timeout
            log.info(
                "record use_airtest max_time: {}, bit_rate: {}".format(
                    max_time, bit_rate
                )
            )
            self.dev.start_recording(max_time, bit_rate)
        else:
            if not self.support:
                return
            self.reset()
            device_id = gr.get_device_id()
            screen_size = gr.get_device_size()
            if not bit_rate:
                bit_rate = screen_size[0] * screen_size[1] * bit_rate_level
            if ":" in device_id:
                cmd = "{} -s {} shell screenrecord --bugreport " \
                      "--size {}x{} --time-limit {} --bit-rate {} " \
                      "--verbose sdcard/flybirds.mp4".format(airtest_adb_path,
                                                             device_id,
                                                             screen_size[0],
                                                             screen_size[1],
                                                             timeout, bit_rate)
            else:
                cmd = "{} -s {} shell screenrecord" \
                      " --size {}x{} " \
                      "--time-limit {} " \
                      "--bit-rate {}" \
                      " --verbose " \
                      "sdcard/flybirds.mp4".format(airtest_adb_path, device_id,
                                                   screen_size[0],
                                                   screen_size[1], timeout,
                                                   bit_rate)

            proc = cmd_helper.create_sub_process(cmd)
            message = ""
            try:
                out, err = proc.communicate(timeout=10)
                if out:
                    message = "Recording start {}".format(
                        out.decode(encoding="UTF-8")
                    )
                if err:
                    message = "Recording start error {}".format(
                        err.decode(encoding="UTF-8")
                    )
            except Exception as e:
                message = "Recording start /sdcard/flybirds.mp4 , " \
                          f"innerError:{str(e)}"
            log.debug("start_record message: {}".format(message))
            time.sleep(1)
            self.process = proc
            self.status = 2
            self.start_time = time.time()

    def stop_record(self):
        """
        Stop the current screen recording
        """
        log.info("stop_record")
        if self.use_airtest_record:
            log.info("stop_record use_airtest_record")
            self.dev.stop_recording(output=self.output_file)
            self.dev.adb.pull(self.recording_file, self.output_file)
        else:
            if not self.support:
                return
            proc = self.process
            if proc is None:
                return "No recording service"

            message = ""
            proc.wait()
            proc_code = proc.poll()
            if proc_code is None:
                proc.terminate()
                time.sleep(2)
                message = "Stop recording"
            else:
                message = "Recording is over，code={}".format(proc_code)
            self.end_time = time.time()
            self.status = 3
            out, err = proc.communicate()
            if out:
                message += " Screen recording {}".format(
                    out.decode(encoding="UTF-8")
                )
                log.debug(
                    "stop_record out message: {}, proc_code: {}".format(
                        message, str(proc_code)
                    )
                )
            if err:
                message += " Screen recording error {}".format(
                    err.decode(encoding="UTF-8")
                )
                log.warn(
                    "stop_record err "
                    "ScreenRecordException: {}, proc_code: {}".format(
                        message, str(proc_code)
                    )
                )
                raise ScreenRecordException(message)
            log.info(
                "stop_record message: {}, proc_code: {}".format(
                    message, str(proc_code)
                )
            )
            return message

    def copy_record(self, save_path):
        """
        Copy the screen recording from the mobile phone to the current client
        """
        device_id = gr.get_device_id()
        cmd = "{} -s {} pull {} {}".format(airtest_adb_path,
                                           device_id, self.recording_file,
                                           save_path
                                           )
        # sub process start time
        start_time = datetime.datetime.now()
        proc = cmd_helper.create_sub_process(cmd)
        message = ""
        try:
            proc.communicate(timeout=30)
        except Exception as e:
            message = "cp screenshot {} not end " \
                      "in 30 seconds," \
                      " innerError:{}".format(self.recording_file, str(e))
        proc_code = proc.poll()

        # time out
        while proc.poll() is None:
            end_time = datetime.datetime.now()
            diff = end_time - start_time
            if diff.seconds > (60 * 2):
                log.warn("copy record running timeout")
                if proc.poll() is None:
                    # proc_code=5 means kill sub process
                    proc_code = 5
                break

        if int(proc_code) == 0:
            log.info("copy record success")
        else:
            log.warn(
                "copy_record not copy_success message: {}".format(message)
            )
            if proc is not None:
                proc.terminate()

            raise ScreenRecordException(message)

        proc.terminate()

    def clear_record(self):
        """
        Delete the existing screen recording file
        """
        device_id = gr.get_device_id()
        cmd = "{} -s {} shell rm -r {}".format(airtest_adb_path, device_id,
                                               self.recording_file)
        proc = cmd_helper.create_sub_process(cmd)
        # message = ""
        try:
            proc.communicate(timeout=15)
        except Exception as e:
            log.error(
                "Screen record deletion {} not end in "
                "15 seconds, innerError:{}".format(
                    self.recording_file, str(e)
                )
            )
        proc_code = proc.poll()
        if int(proc_code) == 0:
            log.info("clear record success")
        proc.terminate()


def link_record(scenario, step_index):
    """
    Associate screenshots to report
    """
    screen_record = gr.get_value("screenRecord")
    log.info(
        "link_record support: {}, step_index: {},"
        " len(scenario.steps): {}, if: {}".format(
            str(screen_record.support),
            str(step_index),
            str(len(scenario.steps)),
            str(len(scenario.steps) > step_index >= 0),
        )
    )
    if not screen_record.support:
        data = "embeddingsTags, stepIndex={}, " \
               "<label>the device does not " \
               "support screen recording</label>".format(step_index)
        scenario.description.append(data)
        return
    feature_name = file_helper.valid_file_name(scenario.feature.name)
    scenario_name = file_helper.valid_file_name(scenario.name)
    if len(scenario.steps) > step_index >= 0:
        file_name = (
                scenario_name
                + uuid_helper.create_short_uuid()
                + str(int(round(time.time() * 1000)))
                + ".mp4"
        )
        screen_shot_dir = gr.get_screen_save_dir()
        if not (screen_shot_dir is None):
            current_screen_dir = os.path.join(screen_shot_dir, feature_name)
        else:
            current_screen_dir = os.path.join(feature_name)
        file_helper.create_dirs_path_object(current_screen_dir)

        src_path = "../screenshot/{}/{}".format(feature_name, file_name)
        data = (
            'embeddingsTags, stepIndex={}, <video controls width="375">'
            '<source src="{}" type="video/mp4"></video>'.format(
                step_index, src_path
            )
        )
        scenario.description.append(data)
        src_path = os.path.join(current_screen_dir, file_name)
        log.info(
            f'default screen_record [link_record] src_path: {src_path}')
        screen_record.copy_record(src_path)



# -*- coding: utf-8 -*-
"""
global plugin holder
"""
import operator
from threading import local

from flybirds.core.plugin.plugin_proxy import PluginProxy


class GlobalContext:
    """
    global plugin context, init on behave hook event
    """

    def __init__(self):
        pass

    device = PluginProxy()
    page = PluginProxy()
    element = PluginProxy()
    app = PluginProxy()
    step = PluginProxy()
    screen = PluginProxy()
    screen_record = PluginProxy()
    ui_driver = PluginProxy()
    interception = PluginProxy()
    platform = None
    config = None
    config_processor = []
    before_run_processor = []
    after_run_processor = []
    before_feature_processor = []
    after_feature_processor = []
    before_scenario_processor = []
    after_scenario_processor = []
    before_step_processor = []
    after_step_processor = []
    before_ui_driver_processor = []
    after_ui_driver_processor = []
    before_tag_processor = []
    after_tag_processor = []
    plugin_processor = []
    language = "zh-CN"
    device_driver = None
    ui_driver_instance = None
    ocr_driver_instance = None
    active_plugin = "default"
    connector = None
    current_local = local()
    ocr_result = []
    image_size = []

    @classmethod
    def set_current_language(cls, l_g):
        cls.current_local.language = l_g

    @classmethod
    def get_current_language(cls):
        return cls.current_local.language

    @classmethod
    def del_current_language(cls):
        c_l = cls.current_local
        del c_l.language

    @staticmethod
    def process(processors_name, *args):
        """
        processor executor
        """
        processors = getattr(GlobalContext, processors_name)
        if processors is not None and len(processors) > 0:
            sort_key = operator.attrgetter("order")
            processors.sort(key=sort_key)
            for processor in processors:
                if hasattr(processor, "can"):
                    can_run = processor.can(*args)
                    if can_run is True:
                        processor.run(*args)
                else:
                    processor.run(*args)

    @staticmethod
    def join(processors_name, processor, replace=0):
        """
        add processor to plugin glob processor, where replace is 1 new
        processor will replace the older
        """
        indx = -1
        if processor is not None and processor.name is not None:
            processors = getattr(GlobalContext, processors_name)
            for i, item in enumerate(processors):
                if item.name is not None and item.name == processor.name:
                    indx = i
                    break
            if replace == 0:
                if indx >= 0:
                    return indx
            elif indx >= 0:
                processors[indx] = processor
                return indx
            processors.append(processor)

        return indx

    @staticmethod
    def insert(processors_name, processor, replace=0):
        """
        insert into processor at first one place and  will remove old one
        when the replace is 1
        """
        indx = -1
        if processor is not None and processor.name is not None:
            processors = getattr(GlobalContext, processors_name)
            for i, item in enumerate(processors):
                if item.name is not None and item.name == processor.name:
                    indx = i
                    break
            if replace == 0:
                if indx >= 0:
                    return indx
            elif indx >= 0:
                del processors[indx]
            processors.insert(0, processor)

        return indx



# -*- coding: utf-8 -*-
"""
web step implements class
"""

import flybirds.core.global_resource as gr
import flybirds.core.plugin.plugins.default.step.common as step_common
import flybirds.utils.flybirds_log as log
from flybirds.core.plugin.plugins.default.step.app \
    import to_app_home, app_login, app_logout
from flybirds.core.plugin.plugins.default.step.record import \
    stop_screen_record
from flybirds.core.plugin.plugins.default.web.interception import \
    Interception as request_op

__open__ = ["Step"]


class Step:
    """Web Step Class"""

    name = "web_step"

    @classmethod
    def jump_to_page(cls, context, param):
        # plugin_page = g_context.page
        # page = plugin_page()
        page = gr.get_value("plugin_page")
        page.navigate(context, param)

    @classmethod
    def return_pre_page(cls, context):
        page = gr.get_value("plugin_page")
        page.return_pre_page(context)

    @classmethod
    def sleep(cls, context, param):
        page = gr.get_value("plugin_page")
        page.sleep(context, param)

    @classmethod
    def screenshot(cls, context):
        step_common.screenshot(context)

    @classmethod
    def prev_fail_scenario_relevance(cls, context, param1, param2):
        """
        Related operations for the previous failure scenario
        """
        step_common.prev_fail_scenario_relevance(context, param1, param2)

    @classmethod
    def stop_screen_record(cls, context):
        log.info('web Step stop_screen_record.')
        stop_screen_record(context)

    @classmethod
    def unblock_page(cls, context):
        return True

    @classmethod
    def cur_page_is(cls, context, param):
        page = gr.get_value("plugin_page")
        page.cur_page_equal(context, param)

    @classmethod
    def has_page_changed(cls, context):
        return True

    @classmethod
    def click_ele(cls, context, selector):
        ele = gr.get_value("plugin_ele")
        ele.ele_click(context, selector)

    @classmethod
    def click_text(cls, context, selector):
        ele = gr.get_value("plugin_ele")
        ele.click_text(context, selector)

    @classmethod
    def click_coordinates(cls, context, x, y):
        ele = gr.get_value("plugin_ele")
        ele.click_coordinates(context, x, y)

    @classmethod
    def ele_text_container(cls, context, selector, param_2):
        ele = gr.get_value("plugin_ele")
        ele.ele_text_include(context, selector, param_2)

    @classmethod
    def wait_text_exist(cls, context, selector):
        ele = gr.get_value("plugin_ele")
        ele.find_text(context, selector)

    @classmethod
    def text_not_exist(cls, context, selector):
        ele = gr.get_value("plugin_ele")
        ele.find_no_text(context, selector)

    @classmethod
    def ele_text_equal(cls, context, selector, param_2):
        ele = gr.get_value("plugin_ele")
        ele.ele_text_equal(context, selector, param_2)

    @classmethod
    def exist_ele(cls, context, selector):
        ele = gr.get_value("plugin_ele")
        ele.ele_exist(context, selector)

    @classmethod
    def wait_ele_exit(cls, context, selector):
        ele = gr.get_value("plugin_ele")
        ele.ele_exist(context, selector)

    @classmethod
    def ele_not_exit(cls, context, selector):
        ele = gr.get_value("plugin_ele")
        ele.ele_not_exist(context, selector)

    @classmethod
    def wait_ele_appear(cls, context, selector):
        """
         page rendering complete appears element[{param}]
        """
        ele = gr.get_value("plugin_ele")
        ele.wait_for_ele(context, selector)

    @classmethod
    def ele_input(cls, context, selector, param_2):
        ele = gr.get_value("plugin_ele")
        ele.ele_input_text(context, selector, param_2)

    @classmethod
    def ele_clear_input(cls, context, selector, param_2):
        ele = gr.get_value("plugin_ele")
        ele.clear_and_input(context, selector, param_2)

    @classmethod
    def ele_swipe(cls, context, selector, param_2, param_3):
        ele = gr.get_value("plugin_ele")
        ele.ele_slide(context, selector, param_2, param_3)

    @classmethod
    def full_screen_swipe(cls, context, param_1, param_2):
        ele = gr.get_value("plugin_ele")
        ele.full_screen_slide(context, param_1, param_2)

    @classmethod
    def ele_select(cls, context, selector, param_2):
        ele = gr.get_value("plugin_ele")
        ele.ele_select(context, selector, param_2)

    @classmethod
    def full_screen_swipe_to_ele_aaa(cls, context, param_1, selector):
        """
        from {param1} find[{param2}]element
        """
        ele = gr.get_value("plugin_ele")
        ele.find_full_screen_slide(context, param_1, selector)

    @classmethod
    def ele_attr_equal(cls, context, selector, param2, param3):
        ele = gr.get_value("plugin_ele")
        ele.is_ele_attr_equal(context, selector, param2, param3)

    @classmethod
    def text_attr_equal(cls, context, selector, param2, param3):
        ele = gr.get_value("plugin_ele")
        ele.is_text_attr_equal(context, selector, param2, param3)

    @classmethod
    def find_child_from_parent(cls, context, p_selector, c_selector):
        ele = gr.get_value("plugin_ele")
        ele.is_parent_exist_child(context, p_selector, c_selector)

    @classmethod
    def find_text_from_parent(cls, context, p_selector, c_selector, param3):
        ele = gr.get_value("plugin_ele")
        ele.find_text_from_parent(context, p_selector, c_selector, param3)

    @classmethod
    def swipe_to_ele(cls, context, p_selector, param2, c_selector):
        ele = gr.get_value("plugin_ele")
        ele.is_parent_exist_child(context, p_selector, c_selector)

    @classmethod
    def to_app_home(cls, context):
        to_app_home(context)

    @classmethod
    def app_login(cls, context, param1, param2):
        app_login(context, param1, param2)

    @classmethod
    def app_logout(cls, context):
        app_logout(context)

    # -------------------------------------------------------------------------
    # request interception
    # -------------------------------------------------------------------------
    @staticmethod
    def add_request_body(context, service_str):
        request_op.add_some_interception_request_body(service_str)

    @staticmethod
    def remove_request_body(context, service_str):
        request_op.remove_some_interception_request_body(service_str)

    @staticmethod
    def clear_all_request_body(context):
        request_op.clear_interception_request_body()

    # -------------------------------------------------------------------------
    # request service listening
    # -------------------------------------------------------------------------
    @staticmethod
    def add_request_mock(context, service_str, mock_case_id_str):
        request_op.add_some_interception_mock(service_str, mock_case_id_str)

    @staticmethod
    def remover_request_mock(context, service_str):
        request_op.remove_some_interception_mock(service_str)

    @staticmethod
    def clear_all_request_mock(context):
        request_op.clear_interception_mock()

    # -------------------------------------------------------------------------
    # compare service requests
    # -------------------------------------------------------------------------
    @staticmethod
    def request_compare_from_path(context, operation, target_data_path):
        request_op.request_compare(operation, target_data_path)

    @staticmethod
    def request_query_str_compare_from_path(context, operation,
                                            target_data_path):
        request_op.request_query_string_compare(operation, target_data_path)

    @staticmethod
    def request_compare_value(context, operation, target_json_path,
                              expect_value):
        request_op.request_compare_value(operation, target_json_path,
                                         expect_value)



# -*- coding: utf-8 -*-
"""
android device core api implement.
"""
from airtest.core.api import (connect_device, shell)

__open__ = ["Device"]


class Device:
    """Android Device Class"""

    name = "android_device"

    def device_connect(self, device_id):
        """
        Initialize device with uri, and set as current device.
        """
        dev = connect_device("Android:///" + device_id)
        return dev

    def use_shell(self, cmd):
        """
        Start remote shell in the target device and execute the command
        :platforms: Android
        """
        return shell(cmd)



# -*- coding: utf-8 -*-
"""
app step implement
"""
import flybirds.core.global_resource as gr
import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_ele as poco_ele
import flybirds.utils.flybirds_log as log
from flybirds.core.global_context import GlobalContext as g_Context


def init_device(context, param=None):
    device_id = param
    g_Context.device.device_connect(device_id)
    gr.set_value("deviceid", device_id)
    log.info("device connected:{}".format(device_id))

    # Get the globally defined poco object
    poco_instance = g_Context.element.ui_driver_init()
    gr.set_value("pocoInstance", poco_instance)
    context.poco_instance = poco_instance
    log.info("poco initial complete")


def connect_device(context, param):
    g_Context.device.device_connect(param)


def install_app(context, param):
    g_Context.app.install_app(param)


def uninstall_app(context, param):
    g_Context.app.uninstall_app(param)


def start_app(context, param):
    g_Context.app.wake_app(param, 10)
    gr.set_value("packageName", param)
    # Modal box error detection
    poco_ele.detect_error()


def restart_app(context):
    package_name = gr.get_app_package_name()
    g_Context.app.shut_app(package_name)
    wait_time = gr.get_frame_config_value("app_start_time", 6)
    g_Context.app.wake_app(package_name, wait_time)


def stop_app(context):
    package_name = gr.get_app_package_name()
    g_Context.app.shut_app(package_name)


def return_pre_page(context):
    g_Context.element.key_event("4")


def to_app_home(context):
    schema_goto_module = gr.get_value("projectScript").custom_operation
    to_home = getattr(schema_goto_module, "to_home")
    to_home()


def app_login(context, param1, param2):
    schema_goto_module = gr.get_value("projectScript").custom_operation
    login = getattr(schema_goto_module, "login")
    login(param1, param2)


def app_logout(context):
    schema_goto_module = gr.get_value("projectScript").custom_operation
    logout = getattr(schema_goto_module, "logout")
    logout()



# -*- coding: utf-8 -*-
"""
when run behave feature will trigger the before and after event
"""
from flybirds.core.global_context import GlobalContext
from flybirds.utils import launch_helper


class OnBefore:  # pylint: disable=too-few-public-methods
    """
    feature before processor
    """

    name = "OnBefore"
    order = 5

    @staticmethod
    def can(context, feature):
        return True

    @staticmethod
    def run(context, feature):
        """
        run before processor and run user defined hook func
        """
        # thread local set feature language
        GlobalContext.set_current_language(feature.language)

        # hook extend by tester
        before_feature_extend = launch_helper.get_hook_file(
            "before_feature_extend"
        )
        if before_feature_extend is not None:
            before_feature_extend(context, feature)


class OnAfter:  # pylint: disable=too-few-public-methods
    """
    feature after processor
    """

    name = "OnAfter"
    order = 100

    @staticmethod
    def can(context, feature):
        return True

    @staticmethod
    def run(context, feature):
        """
        run after processor and run user defined hook func
        """
        # thread local release feature language
        GlobalContext.del_current_language()
        # hook extend by tester
        after_feature_extend = launch_helper.get_hook_file(
            "after_feature_extend"
        )
        if after_feature_extend is not None:
            after_feature_extend(context, feature)


# register event to global processor wait to be triggered
var = GlobalContext.join("before_feature_processor", OnBefore, 1)
var1 = GlobalContext.join("after_feature_processor", OnAfter, 1)



# -*- coding: utf-8 -*-
# @Time : 2022/3/22 17:12
# @Author : hyx
# @File : active_tag.py
# @desc : active_tag
import sys

import six
from behave.tag_matcher import ActiveTagMatcher, setup_active_tag_values

import flybirds.core.global_resource as gr
from flybirds.core.global_context import GlobalContext
from flybirds.utils import flybirds_log as log


def bool_to_string(value):
    """Converts a boolean active-tag value into its normalized
    string representation.

    :param value:  Boolean value to use (or value converted into bool).
    :returns: Boolean value converted into a normalized string.
    """
    return str(bool(value)).lower()


# -----------------------------------------------------------------------------
# DEFAULT SUPPORTED: ACTIVE-TAGS
# -----------------------------------------------------------------------------
def default_active_tag_value_provider():
    platform = GlobalContext.platform if GlobalContext.platform is not None \
        else 'android'
    cur_browser = gr.get_value("cur_browser", 'chromium')
    log.info(f'default_active_tag_provider :{platform} {cur_browser}')
    return {
        "python2": bool_to_string(six.PY2),
        "python3": bool_to_string(six.PY3),
        "os": sys.platform.lower(),
        "platform": platform,
        "deviceType": 'ivd',
        "cur_browser": cur_browser,
    }


def active_tag_init():
    """
      # -- MATCHES ANY TAGS: @use.with_{category}={value}
     # NOTE:
         active_tag_value_provider provides category values for active tags.
     """
    # get custom active_tag
    # tag_provider_module = gr.get_value("projectScript").tag_provider
    # tag_value_provider = getattr(tag_provider_module,
    #                              "ACTIVE_TAG_VALUE_PROVIDER")
    # if tag_value_provider is None:
    #     tag_value_provider = {}
    # log.info(f'tag_value_provider :{tag_value_provider}')

    # get default active_tag
    default_active_tag = default_active_tag_value_provider().copy()
    if default_active_tag is None:
        default_active_tag = {}
    active_tag_value_provider = default_active_tag
    active_tag_matcher = ActiveTagMatcher(active_tag_value_provider)
    gr.set_value("active_tag_matcher", active_tag_matcher)
    return active_tag_value_provider


def merge(dict1, dict2):
    res = {**dict1, **dict2}
    return res


class OnBeforeAll:
    """
    prepare
    """
    name = "OnBeforeAll"
    order = 0

    @staticmethod
    def can(context):
        return True

    @staticmethod
    def run(context):
        """
          SETUP ACTIVE-TAG MATCHER (with user_data)
          USE: behave -D browser=safari ...
        """
        log.info(f'[active_before_all] user_data:{context.config.userdata}')
        active_tag_value_provider = active_tag_init()
        log.info(f'[active_before_all] active_tag_provider:'
                 f'{active_tag_value_provider}')
        setup_active_tag_values(active_tag_value_provider,
                                context.config.userdata)


class OnBeforeFeature:
    """
    prepare OnBeforeFeature
    """
    name = "OnBeforeFeature"
    order = 5

    @staticmethod
    def can(context, feature):
        return True

    @staticmethod
    def run(context, feature):
        log.info(f'[before_feature] feature.tags:{feature.tags}')
        active_tag_matcher = gr.get_value("active_tag_matcher")
        if active_tag_matcher.should_exclude_with(feature.tags):
            feature.skip(reason=active_tag_matcher.exclude_reason)


class OnBeforeScenario:
    """
    before run scenario will trigger this
    """
    name = "OnBeforeScenario"
    order = 5

    @staticmethod
    def can(context, scenario):
        return True

    @staticmethod
    def run(context, scenario):
        log.info(
            f'[before_scenario] scenario.effective_tags:'
            f'{scenario.effective_tags}')
        active_tag_matcher = gr.get_value("active_tag_matcher")
        # -- NOTE: scenario.effective_tags := scenario.tags + feature.tags
        if active_tag_matcher.should_exclude_with(scenario.effective_tags):
            scenario.skip(reason=active_tag_matcher.exclude_reason)


var_all = GlobalContext.join("before_run_processor", OnBeforeAll, 1)
var_2 = GlobalContext.join("before_feature_processor", OnBeforeFeature, 1)
var_3 = GlobalContext.join("before_scenario_processor", OnBeforeScenario, 1)



# -*- coding: utf-8 -*-
"""
log helper
"""
import logging

# create logger
logger = logging.getLogger("flybirds_log")
logger.setLevel(logging.DEBUG)

# create console handler and set level to debug
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)

# create formatter
formatter = logging.Formatter(
    "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
)

# add formatter to ch
ch.setFormatter(formatter)

# add ch to logger
logger.addHandler(ch)


def debug(*args):
    """
    log debug
    """
    for arg in args:
        logger.debug(arg)


def info(*args):
    """
    log info
    """
    for arg in args:
        logger.info(arg)


def warn(*args):
    """
    warn log
    """
    for arg in args:
        logger.warning(arg)


def error(*args):
    """
    error log
    """
    for arg in args:
        logger.error(arg)



# -*- coding: utf-8 -*-
"""
flybirds common error
"""
import flybirds.core.global_resource as gr


class FlybirdNotFoundException(Exception):
    """
    not find flybirds
    """

    def __init__(self, message, select_dic, error=None):
        message = f"selectors={str(select_dic)} {message}"
        if error is not None:
            message = f"{message} innerErr:{error}"
        super().__init__(message)
        self.message = message

    def __str__(self):
        return str(self.message)


class PositionNotChangeException(Exception):
    """
    position not change
    """

    def __init__(self, message):
        super().__init__(message)
        self.message = message

    def __str__(self):
        return str(self.message)


class FlybirdCallMethodParamsException(Exception):
    """
    params error
    """

    def __init__(self, method, param_name):
        message = f"call method:{method} has invalid params:{param_name}"
        super().__init__(message)
        self.message = message

    def __str__(self):
        return str(self.message)


class FlybirdEleExistsException(Exception):
    """
    ele not exists
    """

    def __init__(self, message):
        super().__init__(message)
        self.message = message

    def __str__(self):
        return str(self.message)


class FlybirdVerifyException(Exception):
    """
    verify error
    """

    def __init__(self, message):
        super().__init__()
        self.message = message

    def __str__(self):
        return str(self.message)


class FlybirdPositionChanging(Exception):
    """
    position changing
    """

    def __init__(self, message):
        super().__init__()
        self.message = message

    def __str__(self):
        return str(self.message)


class ScreenRecordException(Exception):
    """
    screen record error
    """

    def __init__(self, message):
        super().__init__()
        self.message = message

    def __str__(self):
        return str(self.message)


class FlybirdsVerifyEleException(Exception):
    """
    timeout error
    """

    def __init__(self, message=None, selector=None):
        super().__init__()
        if message is not None:
            self.message = message
        elif selector is not None:
            self.message = self.print_message(selector)

    def __str__(self):
        return str(self.message)

    @staticmethod
    def print_message(param):
        default_timeout = gr.get_frame_config_value("wait_ele_timeout", 30)
        message = f'Timeout {default_timeout * 1000}ms exceeded.\n'
        message += '=' * 20 + ' logs ' + '=' * 20
        message += f'\nwaiting for selector "{param}"\n'
        message += '=' * 46
        return message


class FlybirdsException(Exception):
    """
     flybirds exception base class
    """

    def __init__(self, message):
        super().__init__()
        self.message = message

    def __str__(self):
        return str(self.message)



# -*- coding: utf-8 -*-
"""
ios device core api implement.
"""
import time
import re
from tidevice import Device as t_device
from tidevice._wdaproxy import WDAService
from airtest.core.api import connect_device
from flybirds.utils.flybirds_log import logger
from flybirds.core.global_context import GlobalContext

__open__ = ["Device"]


class Device:
    """IOS App Class"""

    name = "ios_device"

    def device_connect(self, device_id):
        """
        Initialize device with uri, and set as current device.
        """

        if GlobalContext.connector is not None:
            return GlobalContext.connector.connect(device_id, 60)
        else:
            return PC.static_connect(device_id)

    def pc_init(self, device_id, test_driver, timeout=50):
        """
        please not call multi times
        """

        if GlobalContext.connector is None:
            return PC(device_id,
                      test_driver, timeout)
        else:
            logger.info("already init connector")
            return GlobalContext.connector


class PC:
    """
    pc connector mac windows
    """

    def __init__(self, device_id, test_driver, timeout):
        self.process = None
        self.replay = None
        try:
            d = t_device(device_id)
            self.process = WDAService(d, test_driver)

            # cmds = [
            #     sys.executable, '-m', 'tidevice', '-u', d.udid, 'relay',
            #     '8200', '8100'
            # ]
            # self.replay = subprocess.Popen(cmds, stdout=None, stderr=None,
            #                                shell=True)
            self.process.start()

            time.sleep(1)
            self.wait_ready(timeout)
        except Exception as wda_error:
            self.close()
            logger.info(f"start wda proxy fail:{str(wda_error)}")

    def connect(self, device_id, timeout):
        try:
            dev = None
            if PC.check_ip(device_id):
                dev = connect_device(f"ios:///{device_id}")
            else:
                dev = connect_device(f"ios:///http+usbmux://{device_id}")
            return dev
        except Exception as c_e:
            logger.info(f"connect device fail: {str(c_e)}")
            self.close()

    @staticmethod
    def static_connect(device_id):
        try:
            dev = None
            if PC.check_ip(device_id):
                dev = connect_device(f"ios:///{device_id}")
            else:
                dev = connect_device(f"ios:///http+usbmux://{device_id}")
            return dev
        except Exception as c_e:
            logger.info(f"connect device fail: {str(c_e)}")

    @staticmethod
    def check_ip(ip_addr):
        result = re.findall(r"\b(?:[0-9]{1,3}\.){3}[0-9]{1,3}\b", ip_addr)
        if result:
            return True
        else:
            return False

    def close(self):
        if hasattr(self, "process") and self.process is not None:
            self.process.stop()

        if hasattr(self, "replay") and self.replay is not None:
            self.replay.terminate()

    def wait_ready(self, timeout):
        deadline = time.time() + timeout

        while time.time() <= deadline:
            if hasattr(self, "process") and self.process is not None:
                if self.process._is_alive() is True:
                    break
            time.sleep(1)
            logger.info("device not ready")



# -*- coding: utf-8 -*-
"""
Get element attributes
"""
import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_ele as poco_ele
from flybirds.core.plugin.plugins.default.ui_driver.poco import poco_manage


def get_ele_attr(
        poco,
        selector_str,
        optional,
        attr_name,
        deal_method=None,
        params_deal_module=None,
):
    """
    get the specified attribute of the element, and after the attribute is
    obtained,you can choose whether to process it through a custom method.
    """
    poco_ele.wait_exists(poco, selector_str, optional)
    poco_object = poco_manage.create_poco_object_by_dsl(
        poco, selector_str, optional
    )
    ele_attr = poco_object.attr(attr_name)
    if not (deal_method is None):
        deal_method = getattr(params_deal_module, deal_method)
        ele_attr = deal_method(ele_attr)
    return ele_attr



# -*- coding: utf-8 -*-
"""
when run step will trigger this event
"""

from flybirds.core.global_context import GlobalContext
from flybirds.utils import flybirds_log as log
from flybirds.utils import launch_helper


def step_init(context, step):
    """
    adjust the order of the current steps for use in associated screenshots
    """
    # adjust the order of the current steps for use in associated screenshots
    context.cur_step_index += 1


class OnBefore:  # pylint: disable=too-few-public-methods
    """
    before step
    """

    name = "OnBefore"
    order = 5

    @staticmethod
    def can(context, step):
        return True

    @staticmethod
    def run(context, step):
        """
        exe before step
        """
        log.info(step)
        step_init(context, step)
        log.info(f"run step:{step.name}")
        # if there is a hook custom behavior, call the related function
        before_step_extend = launch_helper.get_hook_file("before_step_extend")
        if before_step_extend is not None:
            before_step_extend(context, step)


class OnAfter:  # pylint: disable=too-few-public-methods
    """
    after step
    """

    name = "OnAfter"
    order = 100

    @staticmethod
    def can(context, step):
        return True

    @staticmethod
    def run(context, step):
        """
        exe after step
        """
        # hook extend by tester
        after_step_extend = launch_helper.get_hook_file("after_step_extend")
        if after_step_extend is not None:
            after_step_extend(context, step)


# join step event to global processor
var = GlobalContext.join("before_step_processor", OnBefore, 1)
var1 = GlobalContext.join("after_step_processor", OnAfter, 1)



# -*- coding: utf-8 -*-
# @Time : 2022/5/16 17:05
# @Author : hyx
# @File : interception.py
# @desc :web request interception related operations
import json
import os
import re
from urllib.parse import parse_qs

from deepdiff import DeepDiff
from jsonpath_ng import parse as parse_path

import flybirds.core.global_resource as gr
import flybirds.utils.flybirds_log as log
from flybirds.core.exceptions import FlybirdsException

__open__ = ["Interception"]

from flybirds.utils import file_helper
from flybirds.utils.file_helper import read_json_data


class Interception:
    """
    web interception impl
    """
    name = "web_interception"

    # -------------------------------------------------------------------------
    # request interception
    # -------------------------------------------------------------------------
    @staticmethod
    def add_some_interception_request_body(service_str):
        if service_str is None:
            log.error(
                '[addSomeInterceptionRequestBody] param can not be none.')
            return
        service_list = service_str.strip().split(',')
        interception_request = gr.get_value('interceptionRequest')

        for service in service_list:
            interception_request[service.strip()] = {}
        gr.set_value('interceptionRequest', interception_request)

    @staticmethod
    def remove_some_interception_request_body(service_str):
        service_list = service_str.strip().split(',')
        interception_request = gr.get_value('interceptionRequest')

        try:
            for service in service_list:
                request_body = interception_request.pop(service.strip())
                log.info(
                    f'remove data cached by request [{service.strip()}]: '
                    f'{request_body}')
        except Exception as e:
            message = f'[removeSomeInterceptionRequestBody]  ' \
                      f'has KeyError! error key: {str(e)}'
            raise FlybirdsException(message)
        gr.set_value('interceptionRequest', interception_request)

    @staticmethod
    def clear_interception_request_body():
        interception_request = gr.get_value('interceptionRequest')
        interception_request.clear()
        gr.set_value('interceptionRequest', interception_request)

    # -------------------------------------------------------------------------
    # request service listening
    # -------------------------------------------------------------------------
    @staticmethod
    def add_some_interception_mock(service_str, mock_case_id_str):
        if service_str is None or mock_case_id_str is None:
            log.error('[addSomeInterceptionMock] param can not be none. ')
            return

        service_list = service_str.strip().split(',')
        mock_case_id_list = mock_case_id_str.strip().split(',')
        if len(service_list) != len(mock_case_id_list):
            message = f"serviceCount[{service_str}] not equal " \
                      f"mockCaseCount[{mock_case_id_str}]"
            raise FlybirdsException(message)

        interception_values = gr.get_value('interceptionValues')
        for i, service in enumerate(service_list):
            interception_values[service.strip()] = mock_case_id_list[i].strip()

        gr.set_value('interceptionValues', interception_values)

    @staticmethod
    def remove_some_interception_mock(service_str):
        service_list = service_str.strip().split(',')
        interception_values = gr.get_value('interceptionValues')

        try:
            for service in service_list:
                case_id = interception_values.pop(service.strip())
                log.info(f'remove mock data [{case_id}] from request '
                         f'[{service.strip()}]')
        except Exception as e:
            message = f'[removeSomeInterceptionMock]  ' \
                      f'has KeyError! error key: {str(e)}'
            raise FlybirdsException(message)
        gr.set_value('interceptionValues', interception_values)

    @staticmethod
    def clear_interception_mock():
        interception_values = gr.get_value('interceptionValues')
        interception_values.clear()
        gr.set_value('interceptionValues', interception_values)

    # -------------------------------------------------------------------------
    # compare service requests
    # -------------------------------------------------------------------------
    @staticmethod
    def request_compare(operation, target_data_path):
        request_info = get_server_request_body(operation)
        actual_request_obj = None
        if request_info is not None and request_info.get('postData'):
            actual_request_obj = request_info.get('postData')
        log.info(f'[request_compare] actualObj:{actual_request_obj}')
        if actual_request_obj is None:
            message = f'[request_compare] not get listener data for ' \
                      f'[{operation}]'
            raise FlybirdsException(message)
        actual_request_obj = json.loads(actual_request_obj)
        file_path = os.path.join(os.getcwd(), target_data_path)
        expect_request_obj = None
        if os.path.exists(file_path):
            expect_request_obj = file_helper.get_json_from_file_path(file_path)
        log.info(f'[request_compare] expectObj:{expect_request_obj}')
        if expect_request_obj is None:
            message = f'[request_compare] cannot get data form path' \
                      f'[{target_data_path}]]'
            raise FlybirdsException(message)
        handle_diff(actual_request_obj, expect_request_obj, operation,
                    target_data_path)

    @staticmethod
    def request_query_string_compare(operation, target_data_path):
        request_info = get_server_request_body(operation)
        actual_request_obj = None
        if request_info is not None and request_info.get('postData'):
            actual_request_obj = request_info.get('postData')
        if actual_request_obj is None:
            message = f'[requestQuerystringCompare] not get listener data ' \
                      f'for [{operation}]'
            raise FlybirdsException(message)
        actual_request_obj = parse_qs(actual_request_obj)

        file_path = os.path.join(os.getcwd(), target_data_path)
        expect_request_obj = None
        if os.path.exists(file_path):
            expect_request_obj = file_helper.read_file_from_path(file_path)
        if expect_request_obj is None:
            message = f'[requestQuerystringCompare] cannot get data form ' \
                      f'path [{target_data_path}]'
            raise FlybirdsException(message)
        expect_request_obj = parse_qs(expect_request_obj)
        handle_diff(actual_request_obj, expect_request_obj, operation,
                    target_data_path)

    @staticmethod
    def request_compare_value(operation, target_json_path, expect_value):
        request_info = get_server_request_body(operation)
        json_data = None
        if request_info and request_info.get('postData'):
            json_data = request_info.get('postData')
        if json_data is None:
            message = f'[requestCompareValue] not get listener data for ' \
                      f'[{operation}]'
            raise FlybirdsException(message)
        json_data = json.loads(json_data)
        json_path_expr = parse_path(target_json_path)
        target_values = [match.value for match in
                         json_path_expr.find(json_data)]
        log.info(f'[requestCompareValue] get jsonPathData: {target_values}')
        if len(target_values) == 0:
            message = f'[requestCompareValue] cannot get the value from ' \
                      f'path [{target_json_path}] of [{operation}]'
            raise FlybirdsException(message)
        if str(target_values[0]) != expect_value:
            message = f'value not equal, service [{operation}] request ' \
                      f'parameter [{target_json_path}] actual value:' \
                      f'[{target_values[0]}], but expect value:' \
                      f'[{expect_value}]'
            raise FlybirdsException(message)


def get_server_request_body(service):
    interception_request = gr.get_value('interceptionRequest')
    if interception_request:
        return interception_request.get(service)
    return None


def handle_ignore_node(service):
    exclude_paths = []
    exclude_regex_paths = []
    service_ignore_nodes = gr.get_service_ignore_nodes(service)
    if service_ignore_nodes is None:
        return exclude_paths, exclude_regex_paths
    for item in service_ignore_nodes:
        if 'regex' in item:
            regex_item = item.split('regex:')[-1].strip()
            exclude_regex_paths.append(regex_item)
        else:
            path = 'root'
            for level_item in item.split('.'):
                # identifies whether the item is an array
                level_item = level_item.strip()
                item_is_array = re.search(r"([^\[\]]+)\[(\d+)\]",
                                          level_item) is not None
                item_str = ''
                if item_is_array:
                    property_name = "['" + re.search(r"([^\[\]]+)\[(\d+)\]",
                                                     level_item).group(
                        1) + "']"
                    array_index = ''.join(
                        list(map(
                            lambda x: "[" + x + "]",
                            re.findall(r"\[(\d+)\]", level_item))
                        )
                    )
                    item_str = property_name + array_index
                else:
                    item_str = "['" + level_item + "']"
                path += item_str
            exclude_paths.append(path.strip())
    return exclude_paths, exclude_regex_paths


def handle_diff(actual_request_obj, expect_request_obj, operation,
                target_file_name):
    exclude_paths, exclude_regex_paths = handle_ignore_node(operation)
    ignore_order = gr.get_web_info_value("ignore_order", False)
    # diffs with jsons
    diff = DeepDiff(actual_request_obj, expect_request_obj,
                    ignore_order=ignore_order, verbose_level=2,
                    exclude_paths=exclude_paths,
                    exclude_regex_paths=exclude_regex_paths)
    if diff:
        format_diff = json.dumps(diff, indent=2)
        message = f'Difference when comparing service request ' \
                  f'[{operation}] with [{target_file_name}]. ' \
                  f'\n Difference node:\n {format_diff} \n'
        raise FlybirdsException(message)
    log.info(f'compare the service request [{operation}] with '
             f'[{target_file_name}], the result is the same.')


def get_case_response_body(case_id):
    operation_module = gr.get_value("projectScript").custom_operation
    get_mock_case_body = getattr(operation_module, "get_mock_case_body")
    mock_case_body = get_mock_case_body(case_id)
    if mock_case_body is not None:
        log.info('[get_case_response_body] successfully get mockCaseBody '
                 'from custom operation')
        return mock_case_body
    log.warn('[get_case_response_body] cannot get mockCaseBody from custom '
             'operation. Now try to get from the folder mockCaseData.')
    # read from folder mockCaseData
    mock_data_path = os.path.join(os.getcwd(), "mockCaseData")
    all_mock_data = read_json_data(mock_data_path)
    if all_mock_data.get(case_id):
        log.info('[get_case_response_body] successfully get mockCaseBody '
                 'from folder mockCaseData')
        return all_mock_data.get(case_id)
    log.warn('[get_case_response_body] cannot get mockCaseBody from folder '
             'mockCaseData.')
    return



# -*- coding: utf-8 -*-
"""
Step implement of element attribute.
"""
import flybirds.core.global_resource as gr
import flybirds.utils.dsl_helper as dsl_helper
from flybirds.core.global_context import GlobalContext as g_Context


def text_attr_equal(context, param1, param2, param3):
    """
    text attribute comparison
    """
    poco_instance = gr.get_value("pocoInstance")

    param1_dict = dsl_helper.params_to_dic(param1)
    selector_str = param1_dict["selector"]
    if "fuzzyMatch" in param1_dict.keys():
        selector_str = "textMatches=" + selector_str
    else:
        selector_str = "text=" + selector_str
    optional = {}
    if "timeout" in param1_dict.keys():
        optional["timeout"] = float(param1_dict["timeout"])
    else:
        optional["timeout"] = gr.get_frame_config_value("wait_ele_timeout", 10)

    param2_dict = dsl_helper.params_to_dic(param2, "attrName")

    target_attr = param2_dict["attrName"]

    target_attr_value = param3.strip()

    deal_method = None
    params_deal_module = None
    if "dealMethod" in param2_dict.keys():
        deal_method = param2_dict["dealMethod"]
        params_deal_module = gr.get_value("projectScript").params_deal
    g_Context.element.ele_attr_is(
        poco_instance,
        selector_str,
        optional,
        target_attr,
        target_attr_value,
        deal_method,
        params_deal_module,
    )


def ele_attr_equal(context, param1, param2, param3):
    """
    Element attribute comparison
    """
    poco_instance = gr.get_value("pocoInstance")

    param1_dict = dsl_helper.params_to_dic(param1)
    selector_str = param1_dict["selector"]
    optional = {}
    if "path" in param1_dict.keys():
        optional["path"] = param1_dict["path"]
    elif "multiSelector" in param1_dict.keys():
        optional["multiSelector"] = param1_dict["multiSelector"]
    if "timeout" in param1_dict.keys():
        optional["timeout"] = float(param1_dict["timeout"])
    else:
        optional["timeout"] = gr.get_frame_config_value("wait_ele_timeout", 10)

    param2_dict = dsl_helper.params_to_dic(param2, "attrName")

    target_attr = param2_dict["attrName"]

    target_attr_value = param3.strip()

    deal_method = None
    params_deal_module = None
    if "dealMethod" in param2_dict.keys():
        deal_method = param2_dict["dealMethod"]
        params_deal_module = gr.get_value("projectScript").params_deal

    g_Context.element.ele_attr_is(
        poco_instance,
        selector_str,
        optional,
        target_attr,
        target_attr_value,
        deal_method,
        params_deal_module,
    )



# -*- coding: utf-8 -*-
"""
plugin load and destroy
"""

from flybirds.core.global_context import GlobalContext

__import__("flybirds.core.plugin.loader")


def load(context):
    """
    load plugin
    """
    GlobalContext.process("plugin_processor", context)


def destroy(context):
    """
     destroy plugin
    """
    raise Exception("not implement destroy")



# -*- coding: utf-8 -*-
"""
process args
"""
import base64
import json
import os

import flybirds.utils.flybirds_log as log
import flybirds.utils.uuid_helper as uuid_helper
from flybirds.core.exceptions import FlybirdVerifyException
from flybirds.core.tag_expression import TagExpression
from flybirds.utils import file_helper


def default_report_path():
    """
    Generate the default report directory
    """
    report_path = "report{}{}{}report.json".format(
        os.sep, uuid_helper.create_uuid(), os.sep
    )
    return report_path


def check_workspace_args(feature_path):
    base_dir = os.getcwd()
    config_dir = os.path.join(base_dir, "config")

    if os.path.isdir(config_dir) is False:
        raise Exception(f"not find config directory in {base_dir}")


def parse_args(
        feature_path, tag, report_format, report_path, define, rerun, es,
        to_html, run_at, processes
):
    """
    process args
    :return:
    """
    log.info(
        f"flybirds cmd info: {feature_path} {tag} {report_format} "
        f"{report_path} {define} {rerun} {es} {to_html} {run_at}"
    )

    check_workspace_args(feature_path)

    use_define = []
    tags = []
    need_rerun = False

    # process define
    if define:
        for define_item in define:
            use_define.append("--define")
            use_item_array = define_item.strip().split("=", 1)
            use_item_key = use_item_array[0]
            use_item_value = use_item_array[1]
            use_item = "{}={}".format(
                use_item_key,
                str(
                    base64.b64encode(use_item_value.encode("utf-8")),
                    "utf-8",
                ),
            )
            use_define.append(use_item)

    # process rerun
    if rerun:
        need_rerun = rerun

    # create and clear report path
    report_dir_path = report_path[0: report_path.rfind(os.sep)]
    file_helper.create_dirs(report_dir_path)
    file_helper.clear_dirs(report_dir_path)
    report_path = "-o {}".format(report_path)
    log.info("report path: {}".format(report_path))

    # args cmd_array
    cmd_array = ["behave", feature_path, report_format, report_path]

    # process es
    if not (es is None):
        cmd_array.extend(
            [
                "--define es={}".format(
                    str(
                        base64.b64encode(
                            json.dumps(es).encode("utf-8")
                        ).decode("utf-8")
                    )
                )
            ]
        )

    # process tag
    behave_tag_array = []
    if not (tag is None):
        # tag deal
        tags.append(tag)
        temp_tags = TagExpression(tags)
        tag_array = str(temp_tags).split(" ")
        behave_tag_array = ["--tags={}".format(item) for item in tag_array]
        cmd_array.extend(behave_tag_array)

    # create and clear screenshot ptah
    screen_shot_dir = "{}{}screenshot".format(report_dir_path, os.sep)
    file_helper.create_dirs(screen_shot_dir)
    file_helper.clear_dirs(screen_shot_dir)
    user_data_screen_dir = "screenShotDir={}".format(
        str(base64.b64encode(screen_shot_dir.encode("utf-8")), "utf-8")
    )

    use_define.extend(["--define", user_data_screen_dir])
    log.info("screenshot path: {}".format(screen_shot_dir))

    if run_at is None:
        run_at = 'local'
    run_at_base = str(base64.b64encode(run_at.encode('utf-8')), 'utf-8')
    use_define.extend(["--define", f"run_at={run_at_base}"])

    if len(use_define) > 0:
        cmd_array.extend(use_define)
        has_user_tag_exist(use_define)

    cmd_array.append(
        "--no-color --no-capture --no-capture-stderr"
    )
    cmd_str = " ".join(cmd_array)
    log.info("the assembled behave execution command: {}".format(cmd_str))
    return {
        "cmd_str": cmd_str,
        "need_rerun": need_rerun,
        "report_dir_path": report_dir_path,
        "use_define": use_define,
        "env_config": es,
        "report_format": report_format,
        "html": to_html,
        "run_at": run_at,
        "processes": processes,
        "feature_path": feature_path,
        "parsed_tags": behave_tag_array
    }


def has_user_tag_exist(arr):
    new_arr = []
    for item in arr:
        if not ('=' in item):
            continue
        key = item.split("=", 1)[0]
        if key in new_arr:
            message = f"the tag [{key}] already exist,please chang other one."
            raise FlybirdVerifyException(message)
        new_arr.append(item.split("=", 1)[0])



# -*- coding: utf-8 -*-
"""
This module defines the steps related to the app.
"""
from behave import step

from flybirds.core.global_context import GlobalContext as g_Context
from flybirds.utils.dsl_helper import ele_wrap


@step("install app[{selector}]")
@ele_wrap
def install_app(context, selector=None):
    g_Context.step.install_app(context, selector)


@step("delete app[{selector}]")
@ele_wrap
def uninstall_app(context, selector=None):
    """
    uninstall app
    """
    g_Context.step.uninstall_app(context, selector)


@step("start app[{selector}]")
@ele_wrap
def start_app(context, selector=None):
    g_Context.step.start_app(context, selector)


@step("restart app")
def restart_app(context):
    g_Context.step.restart_app(context)


@step("close app")
def stop_app(context):
    g_Context.step.stop_app(context)



# -*- coding: utf-8 -*-
"""
web screen record
"""

import flybirds.core.global_resource as gr
import flybirds.utils.flybirds_log as log

__open__ = ["ScreenRecordInfo"]


class ScreenRecordInfo:
    name = "web_screen_record"
    instantiation_timing = "plugin"

    def __init__(self):
        self.support = True
        self.start_time = None
        self.end_time = None
        self.process = None
        # 0 Just created 1 Reset state 2 Start recording
        self.status = 0

    @staticmethod
    def stop_record():
        log.info("[web stop_record] start")
        page_obj = gr.get_value("plugin_page")
        if page_obj is None or (not hasattr(page_obj, 'context')):
            log.error('[web stop_record] get page object has error!')

        page_obj.context.close()

    @staticmethod
    def copy_record(src_path):
        page_obj = gr.get_value("plugin_page")
        if page_obj is None or (not hasattr(page_obj, 'page')):
            log.error('[web copy_record] get page object has error!')

        video = page_obj.page.video
        path = video.path()
        log.info(f'[web copy_record] web_record path: {path}')
        # Save video as
        video.save_as(src_path)
        video.delete()



# -*- coding: utf-8 -*-
"""
package search
"""
import os
from flybirds.utils.flybirds_log import logger
from flybirds.core.launch_cycle.run_manage import RunManage
from flybirds.utils.pkg_helper import find_package


class PackageFinder:
    """
    find extend package
    """

    name = "PackageFinder"
    order = 12

    @staticmethod
    def find_package(context):
        if context.__contains__("pkg_query") and context.get(
                "pkg_query") is not None and context.get("pkg_query") != "":
            pkg_list = find_package(context.get("pkg_query"))
            return pkg_list
        else:
            return None

    @staticmethod
    def can(context):
        return True

    @staticmethod
    def run(context):
        try:
            logger.info("find extend pkg")
            pkg_query = "-flybirds-plugin"
            context["pkg_query"] = pkg_query
            pkg_list = PackageFinder.find_package(context)
            if pkg_list is not None and len(pkg_list) > 0:
                os.environ["extend_pkg_list"] = ",".join(pkg_list)
        except Exception as find_error:
            logger.info(f"find pack error :{str(find_error)}")


# add event to processor, launch init should be first one
RunManage.insert("before_run_processor", PackageFinder, 1)



# -*- coding: utf-8 -*-
"""
hold global config
"""
import logging

import flybirds.utils.flybirds_log as log

_global_dict = {}


def init_glb():
    """
    initialize global variables
    """
    global _global_dict
    _global_dict = {
        "configManage": None,
        "projectScript": None,
        "userData": {},
        "deviceInstance": None,
        "pocoInstance": None,
        "ocrInstance": None,
        "rerunFailInfo": None,
        "appEnvConfig": None,
        "packageName": None,
        "package_path": None,
        "deviceid": None,
        "platform": None,
        "run_info": None,
        "web_driver_agent": None,
        "playwright": None,
        "browser": None,
        "plugin_page": None,
        "interceptionRequest": {},
        "interceptionValues": {}
    }


def set_value(key, value):
    """
    update or add global attributes
    """
    global _global_dict
    _global_dict[key] = value


def get_value(key, def_value=None):
    """
    get global attribute value
    """
    if key in _global_dict.keys():
        value = _global_dict[key]
        if value is not None:
            return value
    return def_value


def get_frame_config_value(key, def_value=None):
    """
    Get a value in the frame_info.json configuration file
    """
    if hasattr(_global_dict["configManage"].frame_info, key):
        value = getattr(_global_dict["configManage"].frame_info, key)
        if value is not None:
            return value
    return def_value


def get_app_config_value(key, def_value=None):
    """
    get a value in the app_info.json configuration file
    """
    if hasattr(_global_dict["configManage"].app_info, key):
        value = getattr(_global_dict["configManage"].app_info, key)
        if value is not None:
            return value
    return def_value


def get_flow_behave_value(key, def_value=None):
    """
    get a value in the flow_behave.json configuration file
    """
    if hasattr(_global_dict["configManage"].flow_behave, key):
        value = getattr(_global_dict["configManage"].flow_behave, key)
        if value is not None:
            return value
    return def_value


def get_device_id():
    """
    get the id of the device currently in use
    """
    if hasattr(_global_dict["configManage"].device_info, "device_id"):
        value = _global_dict["configManage"].device_info.device_id
        if value is not None:
            return value
    return _global_dict["deviceid"]


def get_web_driver_agent():
    """
    get the id of the device currently in use
    """
    if hasattr(_global_dict["configManage"].device_info,
               "web_driver_agent"):
        value = _global_dict["configManage"].device_info.web_driver_agent
        if value is not None:
            return value
    return _global_dict["web_driver_agent"]


def get_platform():
    """
    get the device platform currently in use
    """
    if hasattr(_global_dict["configManage"].device_info, "platform"):
        value = _global_dict["configManage"].device_info.platform
        if value is not None:
            return value
    return _global_dict["platform"]


def get_device_size(def_value=[1080, 1920]):
    """
    get the schema url of the specified page
    """
    if hasattr(_global_dict["configManage"].device_info, "screen_size"):
        value = _global_dict["configManage"].device_info.screen_size
        if value is not None:
            return value
    return def_value


def get_page_schema_url(page_name):
    """
    get the schema url of the page
    """
    all_schema_url = _global_dict["configManage"].schema_info.all_schema_url
    if all_schema_url is None:
        log.warn("[get_page_schema_url] cannot find schema_url.json file")
        return page_name
    page_url = all_schema_url.get(page_name)
    if page_url is None:
        log.warn(f"the schema_url.json has no schema configuration"
                 f" for [{page_name}]")
        return page_name
    if isinstance(page_url, str):
        return page_url
    platform = get_platform().lower()
    if page_url.get(platform) is None:
        raise Exception(
            f"The [{page_name}] has no schema configuration for the"
            f" [{platform}] platform in schema_url.json")
    return page_url.get(platform)


def get_app_package_name(def_value=None):
    """
    get the package name of the page
    """
    if hasattr(_global_dict["configManage"].app_info, "package_name"):
        value = _global_dict["configManage"].app_info.package_name
        if value is not None:
            return value
    return _global_dict["packageName"]


def get_app_package_path(def_value=None):
    """
    get the schema url of the specified page
    """
    if hasattr(_global_dict["configManage"].app_info, "package_path"):
        value = _global_dict["configManage"].app_info.package_path
        if value is not None:
            return value
    return _global_dict["package_path"]


def get_user_data(key, default_value=None):
    """
    get the data passed in by the user
    """
    return _global_dict["userData"].get(key, default_value)


def update_user_data(key, new_value):
    """
    update the data passed in by the user
    """
    _global_dict["userData"][key] = new_value


def get_screen_save_dir():
    """
    Or the directory where the screenshots are stored
    """
    if hasattr(_global_dict["configManage"].report_info, "screen_shot_dir"):
        return _global_dict["configManage"].report_info.screen_shot_dir
    else:
        return None


def get_log_level():
    """
    get the level of log output
    """
    if "_global_dict" in globals().keys():
        level = _global_dict["configManage"].log_config.level
    else:
        level = "info"
    switcher = {
        "fatal": logging.FATAL,
        "error": logging.ERROR,
        "warn": logging.WARNING,
        "info": logging.INFO,
        "debug": logging.DEBUG,
    }
    return switcher.get(level, logging.INFO)


def get_rerun_info(relevance_id):
    """
    get information about failure scenarios
    including failed screenshots, failed screen recordings,
     feature and scenario names
    """
    return _global_dict["rerunFailInfo"].get(relevance_id, None)


def get_env_config():
    """
    get current environment variables
    """
    return _global_dict["appEnvConfig"]


def get_run_info():
    """
    get run base info
    """
    return _global_dict["run_info"]


def get_web_info_value(key, def_value=None):
    """
    get a value in the web_info configuration
    """
    if hasattr(_global_dict["configManage"].web_info, key):
        value = getattr(_global_dict["configManage"].web_info, key)
        if value is not None:
            return value
    return def_value


def get_ele_locator(key):
    """
    Get the configuration value of the element locator for the current
    runtime platform
    """
    all_locators = _global_dict[
        "configManage"].ele_locator_info.all_ele_locator
    if all_locators is None:
        log.warn("[get_ele_locator] cannot find ele_locator.json file")
        return key
    ele_locator = all_locators.get(key)
    if ele_locator is None:
        log.info(
            f"the ele_locator.json has no element locator configuration "
            f"for [{key}]")
        return key
    if isinstance(ele_locator, str):
        return ele_locator
    platform = get_platform().lower()
    if ele_locator.get(platform) is None:
        raise Exception(
            f"The [{key}] has no element locator configuration for the"
            f" [{platform}] platform in ele_locator.json")
    return ele_locator.get(platform)


def get_service_ignore_nodes(service):
    """
    Get the ignore nodes configuration value of the service interface
    """
    all_ignore_nodes = _global_dict[
        "configManage"].ignore_node_info.all_ignore_nodes
    if all_ignore_nodes is None:
        log.warn("[get_service_ignore_nodes] cannot find configuration value "
                 "from interfaceIgnoreConfig folder")
        return
    service_ignore_nodes = all_ignore_nodes.get(service)
    if service_ignore_nodes is None:
        log.info(f"interface service [{service}] has not set ignore nodes")
    return service_ignore_nodes



# -*- coding: utf-8 -*-
"""
screen proxy
"""
from flybirds.core.global_context import GlobalContext


def screen_shot(path):
    GlobalContext.screen.screen_shot(path)


def screen_link_to_behave(scenario, step_index, tag=None):
    GlobalContext.screen.screen_link_to_behave(scenario, step_index, tag)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
tool setup
"""
from setuptools import setup, find_packages


def parse_requirements(filename):
    """
    get requirements list from config file
    """
    line_iter = (line.strip() for line in open(filename))
    return [line for line in line_iter if line and not line.startswith("#")]


req = parse_requirements("requirements.txt")

setup(
    name="flybirds",
    version="0.3.1",
    author="trip_flight",
    author_email="flybirds_support@trip.com",
    description="BDD-driven natural language automated testing framework",
    long_description="BDD-driven natural language automated testing "
                     "framework, present by Trip Flight",
    keywords=[
        "automation",
        "automated-test",
        "bdd",
        "framework",
        "android",
        "ios",
        "behave",
    ],
    license="MIT license",
    packages=find_packages(exclude=["dist", "build", "tests", "docs"]),
    include_package_data=True,
    install_requires=req,
    entry_points="""
    [console_scripts]
    flybirds = flybirds.launcher:main
    """,
    url="https://github.com/ctripcorp/flybirds",
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)



# -*- coding: utf-8 -*-
"""
base element class
"""
from airtest.core.api import (time, text, keyevent)

import \
    flybirds.core.plugin.plugins.default.ui_driver.poco.poco_attr as poco_attr
import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_click \
    as poco_click
import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_ele as poco_ele
import \
    flybirds.core.plugin.plugins.default.ui_driver.poco.poco_findsnap \
    as find_snap
import \
    flybirds.core.plugin.plugins.default.ui_driver.poco.poco_position \
    as poco_position
import \
    flybirds.core.plugin.plugins.default.ui_driver.poco.poco_selector \
    as poco_selector
import flybirds.core.plugin.plugins.default.ui_driver.poco.poco_swipe \
    as poco_swipe
import \
    flybirds.core.plugin.plugins.default.ui_driver.poco.poco_text as poco_text
import \
    flybirds.core.plugin.plugins.default.ui_driver.poco.poco_verify \
    as poco_verify
from flybirds.core.plugin.plugins.default.ui_driver.poco.poco_input import (
    air_bdd_input
)
from flybirds.core.plugin.plugins.default.ui_driver.poco.poco_manage import (
    create_poco_object_by_dsl
)
from flybirds.core.plugin.plugins.default.ui_driver.poco.poco_screen import (
    air_bdd_screen_size,
)


class BaseElement:
    """Base Element Class"""

    name = "base_element"

    """
    airtest UI  operations
    """

    def str_input(self, input_str, after_input_wait=None):
        """
        Enter a character string, press enter by default after entering
        :param input_str:
        :param after_input_wait:
        :return:
        """
        text(input_str, enter=False)
        if not (after_input_wait is None):
            time.sleep(after_input_wait)

    def key_event(self, key_code):
        """
        Perform key event on the device
        * ``iOS``: Only supports home/volumeUp/volumeDown::
        """
        keyevent(key_code)

    def get_ele_attr(
            self,
            poco,
            selector_str,
            optional,
            attr_name,
            deal_method=None,
            params_deal_module=None,
    ):
        return poco_attr.get_ele_attr(
            poco,
            selector_str,
            optional,
            attr_name,
            deal_method=deal_method,
            params_deal_module=params_deal_module,
        )

    """
       poco ui operations
    """

    def air_bdd_click(
            self,
            poco,
            select_dsl_str,
            optional,
            verify_dsl_str=None,
            verify_optional=None,
            verify_action=None,
    ):
        poco_click.air_bdd_click(
            poco,
            select_dsl_str,
            optional,
            verify_dsl_str=verify_dsl_str,
            verify_optional=verify_optional,
            verify_action=verify_action,
        )

    def verify_click_end(
            self,
            poco,
            verify_dsl_str,
            verify_optional,
            verify_action,
            o_position,
            o_text,
    ):
        poco_click.verify_click_end(
            poco,
            verify_dsl_str,
            verify_optional,
            verify_action,
            o_position,
            o_text,
        )

    def wait_exists(self, poco, selector_str, optional):
        poco_ele.wait_exists(poco, selector_str, optional)

    def not_exist(self, poco, selector_str, optional):
        poco_ele.not_exist(poco, selector_str, optional)

    def wait_disappear(self, poco, selector_str, optional):
        poco_ele.wait_disappear(poco, selector_str, optional)

    def detect_error(self):
        poco_ele.detect_error()

    def find_ele_by_snap(self, poco, config, optional):
        find_snap.find_ele_by_snap(poco, config, optional)

    def verify_ele_by_snap(self, poco, config, optional):
        find_snap.verify_ele_by_snap(poco, config, optional)

    def air_bdd_input(
            self, poco, select_dsl_str, optional, input_str,
            after_input_wait=None
    ):
        air_bdd_input(
            poco,
            select_dsl_str,
            optional,
            input_str,
            after_input_wait=after_input_wait,
        )

    def create_poco_object_by_dsl(self, poco, select_dsl_str, optional):
        return create_poco_object_by_dsl(poco, select_dsl_str, optional)

    def position_change(self, poco, select_str, optional, o_position):
        return poco_position.position_change(poco, select_str, optional,
                                             o_position)

    def position_not_change(
            self, poco, select_str, optional, dur_time, verify_count
    ):
        poco_position.position_not_change(poco, select_str, optional, dur_time,
                                          verify_count)

    def air_bdd_screen_size(self, poco_instance):
        return air_bdd_screen_size(poco_instance)

    def create_poco_object(self, poco, select_dic={}):
        return poco_selector.create_poco_object(poco, select_dic=select_dic)

    def create_parent(self, poco_object):
        return poco_selector.create_parent(poco_object)

    def create_first_child(self, poco_object, select_dic={}):
        return poco_selector.create_first_child(poco_object,
                                                select_dic=select_dic)

    def create_first_offspring(self, poco_object, select_dic={}):
        return poco_selector.create_first_offspring(poco_object,
                                                    select_dic=select_dic)

    def create_first_sibling(self, poco_object, select_dic={}):
        return poco_selector.create_first_sibling(poco_object,
                                                  select_dic=select_dic)

    def select_child(self, poco_object, target_index, select_dic={}):
        return poco_selector.select_child(poco_object, target_index,
                                          select_dic=select_dic)

    def select_offspring(self, poco_object, target_index, select_dic={}):
        return poco_selector.select_offspring(
            poco_object, target_index, select_dic=select_dic
        )

    def select_sibling(self, poco_object, target_index, select_dic={}):
        return poco_selector.select_sibling(poco_object, target_index,
                                            select_dic=select_dic)

    def air_bdd_full_screen_swipe(
            self,
            poco,
            start_point,
            screen_size,
            direction,
            distance,
            duration,
            ready_time=None,
    ):
        poco_swipe.air_bdd_full_screen_swipe(
            poco,
            start_point,
            screen_size,
            direction,
            distance,
            duration,
            ready_time=ready_time,
        )

    def air_bdd_ele_swipe(
            self,
            poco,
            container_dsl_str,
            optional,
            start_point,
            screen_size,
            direction,
            distance,
            duration,
            ready_time=None,
    ):
        poco_swipe.air_bdd_ele_swipe(
            poco,
            container_dsl_str,
            optional,
            start_point,
            screen_size,
            direction,
            distance,
            duration,
            ready_time=ready_time,
        )

    def air_bdd_direction_swipe(
            self, poco, start_point, direction, distance, duration=None
    ):
        poco_swipe.air_bdd_direction_swipe(
            poco, start_point, direction, distance, duration=duration
        )

    def air_bdd_percent_point_swipe(
            self, poco, start_point, end_point, duration=None
    ):
        poco_swipe.air_bdd_percent_point_swipe(
            poco, start_point, end_point, duration=duration
        )

    def full_screen_swipe_search(
            self,
            poco,
            search_dsl_str,
            search_optional,
            swipe_count,
            direction,
            screen_size,
            start_x=None,
            start_y=None,
            distance=None,
            duration=None,
    ):
        poco_swipe.full_screen_swipe_search(
            poco,
            search_dsl_str,
            search_optional,
            swipe_count,
            direction,
            screen_size,
            start_x=start_x,
            start_y=start_y,
            distance=distance,
            duration=duration,
        )

    def air_bdd_swipe_search(
            self,
            poco,
            container_dsl_str,
            container_optional,
            search_dsl_str,
            search_optional,
            swipe_count,
            screen_size,
            direction,
            start_x=None,
            start_y=None,
            distance=None,
            duration=None,
    ):
        poco_swipe.air_bdd_swipe_search(
            poco,
            container_dsl_str,
            container_optional,
            search_dsl_str,
            search_optional,
            swipe_count,
            screen_size,
            direction,
            start_x=start_x,
            start_y=start_y,
            distance=distance,
            duration=duration,
        )

    def get_ele_text_replace_space(
            self,
            poco,
            selector_str,
            optional,
            deal_method_name,
            params_deal_module,
    ):
        return poco_text.get_ele_text_replace_space(
            poco, selector_str, optional, deal_method_name, params_deal_module
        )

    def get_ele_text(
            self,
            poco,
            selector_str,
            optional,
            deal_method_name,
            params_deal_module,
    ):
        return poco_text.get_ele_text(
            poco, selector_str, optional, deal_method_name, params_deal_module
        )

    def text_change(self, poco, select_str, optional, o_text):
        return poco_text.text_change(poco, select_str, optional, o_text)

    def ele_text_is(
            self,
            poco,
            selector_str,
            target_str,
            optional,
            deal_method=None,
            params_deal_module=None,
    ):
        poco_verify.ele_text_is(
            poco,
            selector_str,
            target_str,
            optional,
            deal_method=deal_method,
            params_deal_module=params_deal_module,
        )

    def ele_text_contains(
            self,
            poco,
            selector_str,
            target_str,
            optional,
            deal_method=None,
            params_deal_module=None,
    ):
        poco_verify.ele_text_contains(
            poco,
            selector_str,
            target_str,
            optional,
            deal_method=deal_method,
            params_deal_module=params_deal_module,
        )

    def ele_attr_is(
            self,
            poco,
            selector_str,
            optional,
            target_attr,
            target_attr_value,
            deal_method,
            params_deal_module,
    ):
        poco_verify.ele_attr_is(
            poco,
            selector_str,
            optional,
            target_attr,
            target_attr_value,
            deal_method,
            params_deal_module,
        )



# -*- coding: utf-8 -*-
"""
Device screenshot method.
"""
import os
import time
import traceback
from base64 import b64decode
from PIL import Image

import flybirds.core.global_resource as gr
import flybirds.utils.file_helper as file_helper
import flybirds.utils.flybirds_log as log
import flybirds.utils.uuid_helper as uuid_helper
from flybirds.core.global_context import GlobalContext as g_Context
from flybirds.core.plugin.plugins.default.ios_snapshot import get_screen


class BaseScreen:

    @staticmethod
    def screen_shot(path):
        """
        Take a screenshot and save
        """
        log.info(f"[screen_shot] screen shot start. path is:{path}")
        cur_platform = g_Context.platform
        try:
            if cur_platform is None:
                log.error('[screen_shot] get cur_platform is None!')
                raise Exception("[screen_shot] get cur_platform is None!")

            poco = g_Context.ui_driver_instance
            screen_size = gr.get_device_size()
            if cur_platform.strip().lower() == "ios":
                b64img, fmt = get_screen()
            else:
                b64img, fmt = poco.snapshot(width=screen_size[1])

            open(path, "wb").write(b64decode(b64img))
        except Exception as e:
            log.warn(
                "Screenshot failed path: {}, error: {}".format(path, str(e)),
                traceback.format_exc(),
            )
        log.info("[screen_shot] screen shot end!")

    @staticmethod
    def screen_link_to_behave(scenario, step_index, tag=None):
        """
        screenshot address and linked to the <scr> tag
        The label information is placed in the description of the scene,
        and the json report is processed after all the runs are finished,
        and the <scr> information in the description is converted into
        embeddings information in the step.
        """
        feature_name = file_helper.valid_file_name(scenario.feature.name)
        scenario_name = file_helper.valid_file_name(scenario.name)

        if len(scenario.steps) > step_index >= 0:
            file_name = None
            if not (tag is None):
                file_name = tag
            file_name += (
                    scenario_name
                    + uuid_helper.create_short_uuid()
                    + str(int(round(time.time() * 1000)))
                    + ".png"
            )

            screen_shot_dir = gr.get_screen_save_dir()
            if not (screen_shot_dir is None):
                current_screen_dir = os.path.join(screen_shot_dir,
                                                  feature_name)
            else:
                current_screen_dir = os.path.join(feature_name)
            log.info(f"[screen_link_to_behave] screen_shot_dir path :"
                     f"{screen_shot_dir} and "
                     f"current_screen_dir path: {current_screen_dir}")
            file_helper.create_dirs_path_object(current_screen_dir)

            src_path = "../screenshot/{}/{}".format(feature_name, file_name)
            log.info("[screen_link_to_behave] src_path: {}".format(src_path))
            data = (
                'embeddingsTags, stepIndex={}, <image class ="screenshot"'
                ' width="375" src="{}" />'.format(step_index, src_path)
            )
            scenario.description.append(data)
            screen_path = os.path.join(current_screen_dir, file_name)
            g_Context.screen.screen_shot(screen_path)
            return screen_path

    @staticmethod
    def image_ocr(img_path):
        log.info(f"[image ocr path] image path is:{img_path}")
        ocr = g_Context.ocr_driver_instance
        g_Context.ocr_result = ocr.ocr(img_path, cls=True)
        g_Context.image_size = Image.open(img_path).size
        log.info(f"[image ocr path] image size is:{g_Context.image_size}")
        for line in g_Context.ocr_result:
            log.info(f"[image ocr result] scan line info is:{line}")
            # box = line[0]
            # log.info(f"[image ocr result] scan box info is:{box}")
            # x = (box[0][0] + box[1][0]) / 2
            # y = (box[0][1] + box[2][1]) / 2
            # log.info(f"[image ocr result] scan box xy info is:{x},{y}")
            # txt = line[1][0]
            # log.info(f"[image ocr result] scan txt info is:{txt}")
            # score = line[1][1]
            # log.info(f"[image ocr result] scan score info is:{score}")








# -*- coding: utf-8 -*-

"""
lunch help
"""
import time

import flybirds.core.driver.app as app
import flybirds.core.driver.element as ake
import flybirds.core.global_resource as gr
import flybirds.utils.flybirds_log as log
from flybirds.core.global_context import GlobalContext as g_context


def login():
    """
    use user login
    """
    need_login = gr.get_flow_behave_value("before_run_login", False)
    log.info("before_run_login:{}".format(need_login))
    if need_login:
        schema_goto_module = gr.get_value("projectScript").custom_operation
        login = getattr(schema_goto_module, "login")
        if not (login is None):
            login_user = gr.get_app_config_value("default_user")
            login_password = gr.get_app_config_value("default_password")
            log.info(
                "Login，user：{} password：{}".format(login_user, login_password)
            )
            login(login_user, login_password)


def app_start(page_name):
    """
    device start at init
    """
    page_value = gr.get_flow_behave_value(page_name, None)
    cur_platform = g_context.platform
    if cur_platform.strip().lower() == "web":
        log.info('[app_start] cur_platform is web, run web_start.')
        web_start(page_value)
        return

    device_id = gr.get_device_id()
    package_name = gr.get_app_package_name()
    log.info("device_id:{},".format(device_id))
    log.info("page_name:{},".format(page_name))
    log.info("package_name:{}".format(package_name))
    if not (package_name is None or page_value is None or device_id is None):
        if "restartApp" == page_value:
            app.shut_app(package_name)
            wait_time = gr.get_frame_config_value("app_start_time", 6)
            app.wake_app(package_name, wait_time)
            log.info("complete restartApp and sleep {}".format(wait_time))
        elif "startApp" == page_value:
            wait_time = gr.get_frame_config_value("app_start_time", 6)
            app.wake_app(package_name, wait_time)
            log.info("complete startApp and sleep {}".format(wait_time))
        elif "stopApp" == page_value:
            app.shut_app(package_name)
            log.info("stop app before running")
        elif "backupPage" == page_value:
            ake.key_event("4")


def web_start(page_run_val):
    if page_run_val is None:
        page_run_val = 'restartApp'
    if "restartApp" == page_run_val:
        # determine if the page is closed
        page_obj = gr.get_value("plugin_page")
        if page_obj is not None and hasattr(page_obj, 'context') \
                and page_obj.context is not None:
            log.info(
                '[web_start] Target page, context or browser is opening!')
            return
        re_init_page()
        log.info("complete restart web page")


def get_runtime_data(scenario):
    """
    get run data
    """
    client_id = gr.get_app_config_value("unique_tag")
    device_id = gr.get_device_id()
    run_id = gr.get_app_config_value("run_id")
    build_id = gr.get_app_config_value("build_id")
    pkg_version = gr.get_app_config_value("pkg_version")
    run_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time()))
    location = scenario.feature.location
    data = (
        "client_id: {}, device_id: {}, run_id: {}, build_id: {}"
        ", pkg_version: {}, run_time: {}, location: {}".format(
            client_id,
            device_id,
            run_id,
            build_id,
            pkg_version,
            run_time,
            location,
        )
    )
    return data


def get_hook_file(filename):
    """
    get use hook data
    """
    project_script = gr.get_value("projectScript")
    if hasattr(project_script, "dsl_hook") and hasattr(
            project_script.dsl_hook, filename
    ):
        file_extend = getattr(project_script.dsl_hook, filename)
        return file_extend
    else:
        return None


def re_init_page():
    plugin_page = g_context.page()
    gr.set_value("plugin_page", plugin_page)
    screen_record = g_context.screen_record()
    gr.set_value("screenRecord", screen_record)
    plugin_ele = g_context.element()
    gr.set_value("plugin_ele", plugin_ele)



# -*- coding: utf-8 -*-
"""
Poco manage api
"""
import flybirds.core.global_resource as gr


def ocr_init():
    """
    Initialize the paddleocr object
     :return:
    """
    ocr_lang = gr.get_app_config_value("ocr_lang")
    from paddleocr import PaddleOCR
    # Paddleocr support languages
    # example`ch`, `en`, `fr`, `german`, `korean`, `japan`
    ocr = PaddleOCR(use_angle_cls=True,
                    lang=ocr_lang)  # need to run only once to download and load model into memory

    return ocr




# -*- coding: utf-8 -*-
"""
when behave run scenario will trigger this
"""
import traceback

import flybirds.core.global_resource as gr
import flybirds.utils.language_helper as lge
from flybirds.core.driver import screen
from flybirds.core.global_context import GlobalContext
from flybirds.core.plugin.plugins.default.screen_record import link_record
from flybirds.utils import flybirds_log as log
from flybirds.utils import launch_helper


def scenario_init(context, scenario):
    """
    init description and screen record
    """
    # initialize the description
    # the information added to the description will
    # take effect in this scenario
    log.info('[scenario_init] start!')
    scenario.description.append("initialization description_")
    # Initialize the sequence of steps to be executed
    # which is required for subsequent associated screenshots
    context.cur_step_index = 0
    # Whether to start recording screen
    # it is convenient to have screen recording information
    # associated in the report when it fails
    context.scenario_screen_record = False
    # Restart the app before running
    launch_helper.app_start("before_run_page")
    if gr.get_flow_behave_value("fail_screen_record", True):
        no_screen_record_step = True
        for step in scenario.all_steps:
            if step.name.strip().startswith(
                    lge.parse_glb_str(
                        "start record",
                        scenario.feature.language
                    )
            ) or step.name.strip().startswith(
                lge.parse_glb_str("stop record", scenario.feature.language)
            ):
                no_screen_record_step = False
                break
        if no_screen_record_step:
            try:
                screen_record = gr.get_value("screenRecord")
                if hasattr(screen_record, 'start_record'):
                    timeout = gr.get_flow_behave_value(
                        "scenario_screen_record_time", 120
                    )
                    screen_record.start_record(timeout)
                context.scenario_screen_record = True
            except Exception as scenario_error:
                log.error(
                    f"Running scene: An error occurred when starting to "
                    f"record the screen before {scenario.name}"
                    f", error: {str(scenario_error)}"
                )


def scenario_fail(context, scenario):
    """
    scenario fail handler
    """
    log.info(
        f"[scenario_fail] feature:{scenario.feature.name}, "
        f"scenario:{scenario.name} failed to run"
    )
    need_copy_record = 0

    # the scene fails to output a log and take a screenshot
    for step in scenario.all_steps:
        if step.name.strip().startswith(
                lge.parse_glb_str("start record", scenario.feature.language)
        ):
            need_copy_record += 1
        elif step.name.strip().startswith(
                lge.parse_glb_str("stop record", scenario.feature.language)
        ):
            need_copy_record -= 1
        if step.status == "failed":
            info_log = f"[scenario_fail] step:{step.name}"
            log.info(info_log)
            log.error(f'[scenario_fail] step error msg:{step.error_message}')
            log.info("[scenario_fail] start to do failed screenshot")
            screen.screen_link_to_behave(
                scenario, context.cur_step_index - 1, "fail_"
            )
            break

    # save screen recording
    cur_platform = GlobalContext.platform
    if need_copy_record >= 1 or context.scenario_screen_record \
            or cur_platform.strip().lower() == "web":
        screen_record = gr.get_value("screenRecord")
        screen_record.stop_record()
        link_record(scenario, context.cur_step_index - 1)

    # the processing of the current page after the scene fails
    launch_helper.app_start("scenario_fail_page")
    scenario_screen_record_str = str(context.scenario_screen_record)
    log.info(
        f"scenario_fail, need_copy_record: {str(need_copy_record)},"
        f" context.scenario_screen_record: {scenario_screen_record_str}"
    )


def scenario_success(context, scenario):
    """
    scenario success handler
    """
    # adjustment of the currently displayed page after the scene is successful
    log.info('[scenario_success] start!')
    if context.scenario_screen_record:
        screen_record = gr.get_value("screenRecord")
        screen_record.stop_record()

    launch_helper.app_start("scenario_success_page")


class OnBefore:  # pylint: disable=too-few-public-methods
    """
    before run scenario will trigger this
    """

    name = "OnBefore"
    order = 5

    @staticmethod
    def can(context, scenario):
        return True

    @staticmethod
    def run(context, scenario):
        """
        write run info into description,it will be used at reporter
        """
        try:
            log.info('[scenario_OnBefore] start!')
            f_name = scenario.feature.name
            log.info(
                f"running feature:{f_name}, scenario:{scenario.name},"
                f" location: {scenario.feature.location}"
            )
            scenario_init(context, scenario)
            data = launch_helper.get_runtime_data(scenario)
            scenario.description.append(data)

        except Exception:
            traceback.print_exc()

        # hook extend by tester
        before_scenario_extend = launch_helper.get_hook_file(
            "before_scenario_extend"
        )
        if before_scenario_extend is not None:
            before_scenario_extend(context, scenario)


class OnAfter:  # pylint: disable=too-few-public-methods
    """
    scenario after event
    """

    name = "OnAfter"
    order = 100

    @staticmethod
    def can(context, scenario):
        return True

    @staticmethod
    def run(context, scenario):
        """
        exe scenario after
        """
        try:
            log.info('[scenario_OnAfter] start!')
            if scenario.status == "failed":
                scenario_fail(context, scenario)
            else:
                scenario_success(context, scenario)

            cur_platform = GlobalContext.platform
            if cur_platform.strip().lower() == "web":
                log.info('[web_scenario_OnAfter] reset page、ele、screenRecord')
                gr.set_value("plugin_page", None)
                gr.set_value("screenRecord", None)
                gr.set_value("plugin_ele", None)
        except Exception:
            traceback.print_exc()

        # if there is a hook custom behavior, call the related function
        after_scenario_extend = launch_helper.get_hook_file(
            "after_scenario_extend"
        )
        if after_scenario_extend is not None:
            after_scenario_extend(context, scenario)


# add scenario event to global processor
var = GlobalContext.join("before_scenario_processor", OnBefore, 1)
var1 = GlobalContext.join("after_scenario_processor", OnAfter, 1)



# -*- coding: utf-8 -*-
"""
device prepare
"""
import flybirds.core.global_resource as gr
import flybirds.utils.flybirds_log as log
import flybirds.core.driver.device as device_manage
from flybirds.core.global_context import GlobalContext


class OnPrepare:
    """
    prepare
    """
    name = "On_Device_Prepare"
    order = 15

    @staticmethod
    def init_device(context):
        """
        init device instance
        """
        # get device id
        device_id = gr.get_device_id()
        log.info(f"device_id information:{device_id}")
        if device_id is not None:
            device_instance = device_manage.device_connect(device_id)
            gr.set_value("deviceInstance", device_instance)
            context.device_instance = device_instance
            log.info(f"initialize the device complete:{device_id}")
        else:
            log.info("native device must have device id")
            raise Exception("miss device id")

    @staticmethod
    def can(context):
        """
        only native android can be run
        """
        if gr.get_platform() is not None \
                and (gr.get_platform().lower() == "android"
                     or gr.get_platform().lower()
                     == "ios"):
            return True
        else:
            return False

    @staticmethod
    def run(context):
        """
        install app
        """
        log.info("device prepare")
        OnPrepare.init_device(context)


class OnPCPrepare:
    """
    prepare
    """
    name = "On_PC_Prepare"
    order = 13

    @staticmethod
    def init_device(context):
        """
        prepare device pc connector
        """
        # get device id
        device_id = gr.get_device_id()
        log.info(f"device_id information:{device_id}")
        web_driver_agent = gr.get_web_driver_agent()
        log.info(f"web_driver_agent:{web_driver_agent}")
        if device_id is not None and web_driver_agent is not None and \
                web_driver_agent != "":
            if hasattr(GlobalContext.device, "pc_init"):
                pc_instance = GlobalContext.device.pc_init(device_id,
                                                           web_driver_agent)
                GlobalContext.connector = pc_instance
            log.info(f"initialize the pc connector complete:{device_id}")
        else:
            log.info("ios device must have device id and webDriverAgent")
            raise Exception("miss device id or web_driver_agent")

    @staticmethod
    def can(context):
        """
        only native ios can be run
        """
        if gr.get_platform() is not None \
                and gr.get_platform().lower() == "ios":
            return False
        else:
            return False

    @staticmethod
    def run(context):
        """
        device connect
        """
        log.info("device prepare")
        OnPCPrepare.init_device(context)


# add android prepare into global context
var = GlobalContext.join("before_run_processor", OnPCPrepare, 1)
var1 = GlobalContext.join("before_run_processor", OnPrepare, 1)
